@@ -4,7 +4,7 @@
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-from xpra.codecs.video_enc_pipeline import VideoPipelineHelper
+from xpra.codecs.video_helper import getVideoHelper
 
 
 
@@ -12,15 +12,13 @@ def test_load():
     #NOTE: we assume that the scores:
     # - make nvenc less desirable at HQ (maybe not?)
     # - make cuda csc beat swscale
-    vep = VideoPipelineHelper()
+    vep = getVideoHelper()
     vep.may_init()
 
 
 def main():
     import logging
-    import sys
     logging.root.setLevel(logging.DEBUG)
-    logging.root.addHandler(logging.StreamHandler(sys.stderr))
     test_load()
 
 
