@@ -11,6 +11,7 @@ import gobject
 
 from xpra.log import Logger
 log = Logger()
+log.enable_debug()
 
 from tests.xpra.fake_client import FakeClient
 
@@ -19,7 +20,7 @@ def gl_backing_test(gl_client_window_class=None, w=200, h=100):
     window = gl_client_window_class(FakeClient(), None, 1, 0, 0, w, h, {}, False, {}, 0)
     window.show()
     def update_backing(*args):
-        log.info("update_backing(%s)", args)
+        log("update_backing(%s)", args)
         from xpra.codecs.codec_constants import YUV444P
         import random
         y = chr(int(random.random()*256.0))
@@ -29,7 +30,7 @@ def gl_backing_test(gl_client_window_class=None, w=200, h=100):
         rowstrides = [w, w, w]
         pixel_format = YUV444P
         def update_done(*args):
-            log.info("update_done(%s)", args)
+            log("update_done(%s)", args)
         window._backing.do_gl_paint(0, 0, w, h, img_data, rowstrides, pixel_format, [update_done])
         return True
     def initial_update(*args):
@@ -40,10 +41,6 @@ def gl_backing_test(gl_client_window_class=None, w=200, h=100):
 
 
 def main():
-    import logging
-    logging.basicConfig(format="%(message)s")
-    logging.root.setLevel(logging.DEBUG)
-
     from xpra.client.gl.gl_client_window import GLClientWindow
     gl_backing_test(gl_client_window_class=GLClientWindow)
     gtk.main()
