@@ -1,10 +1,10 @@
 #!/usr/bin/env python
-# This file is part of Parti.
+# This file is part of Xpra.
 # Copyright (C) 2012, 2013 Antoine Martin <antoine@devloop.org.uk>
-# Parti is released under the terms of the GNU GPL v2, or, at your option, any
+# Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-from xpra.shadow_server import take_root_screenshot
+from xpra.server.shadow_server_base import take_root_screenshot
 
 def test_screenshot():
     print("grabbing screenshot")
