@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2010-2013 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2014 Antoine Martin <antoine@devloop.org.uk>
 # Copyright (C) 2008, 2010 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -8,17 +8,20 @@ import signal
 import os
 import sys
 import socket
+import binascii
 from xpra.gtk_common.gobject_compat import import_gobject, import_glib
 gobject = import_gobject()
 
 from xpra.log import Logger
-log = Logger()
+log = Logger("client")
 
-from xpra.net.protocol import Protocol, has_rencode, rencode_version, use_rencode
-from xpra.scripts.config import ENCRYPTION_CIPHERS, python_platform
-from xpra.version_util import version_compat_check, add_version_info
+from xpra.net.protocol import Protocol, use_lz4, use_rencode, get_network_caps
+from xpra.scripts.config import ENCRYPTION_CIPHERS
+from xpra.version_util import version_compat_check, add_version_info, get_platform_info
 from xpra.platform.features import GOT_PASSWORD_PROMPT_SUGGESTION
-from xpra.os_util import get_hex_uuid, get_machine_id, SIGNAMES, strtobytes, bytestostr
+from xpra.platform.info import get_name
+from xpra.os_util import get_hex_uuid, get_machine_id, get_user_uuid, load_binary_file, SIGNAMES, strtobytes, bytestostr
+from xpra.util import typedict, xor
 
 EXIT_OK = 0
 EXIT_CONNECTION_LOST = 1
@@ -31,8 +34,13 @@ EXIT_FAILURE = 7
 EXIT_SSH_FAILURE = 8
 EXIT_PACKET_FAILURE = 9
 EXIT_MMAP_TOKEN_FAILURE = 10
+EXIT_NO_AUTHENTICATION = 11
+EXIT_UNSUPPORTED = 12
+EXIT_REMOTE_ERROR = 13
+
 
 DEFAULT_TIMEOUT = 20*1000
+ALLOW_UNENCRYPTED_PASSWORDS = os.environ.get("XPRA_ALLOW_UNENCRYPTED_PASSWORDS", "0")=="1"
 
 
 class XpraClientBase(object):
@@ -47,12 +55,20 @@ class XpraClientBase(object):
     """
 
     def __init__(self):
+        #this may be called more than once,
+        #skip doing internal init again:
+        if not hasattr(self, "exit_code"):
+            self.defaults_init()
+
+    def defaults_init(self):
         self.exit_code = None
         self.compression_level = 0
-        self.password = None
+        self.display = None
+        self.username = None
         self.password_file = None
         self.password_sent = False
         self.encryption = None
+        self.encryption_keyfile = None
         self.quality = -1
         self.min_quality = 0
         self.speed = 0
@@ -66,19 +82,24 @@ class XpraClientBase(object):
         self._reverse_aliases = {}
         #server state and caps:
         self.server_capabilities = None
+        self._remote_machine_id = None
+        self._remote_uuid = None
         self._remote_version = None
         self._remote_revision = None
         self._remote_platform = None
         self._remote_platform_release = None
         self._remote_platform_platform = None
         self._remote_platform_linux_distribution = None
-        self.make_uuid()
+        self.uuid = get_user_uuid()
         self.init_packet_handlers()
 
     def init(self, opts):
         self.compression_level = opts.compression_level
+        self.display = opts.display
+        self.username = opts.username
         self.password_file = opts.password_file
         self.encryption = opts.encryption
+        self.encryption_keyfile = opts.encryption_keyfile
         self.quality = opts.quality
         self.min_quality = opts.min_quality
         self.speed = opts.speed
@@ -114,12 +135,16 @@ class XpraClientBase(object):
         #overriden in subclasses!
         return "Python"
 
+    def get_scheduler(self):
+        raise NotImplementedError()
+
     def setup_connection(self, conn):
         log.debug("setup_connection(%s)", conn)
-        self._protocol = Protocol(conn, self.process_packet, self.next_packet)
+        self._protocol = Protocol(self.get_scheduler(), conn, self.process_packet, self.next_packet)
         self._protocol.large_packets.append("keymap-changed")
         self._protocol.large_packets.append("server-settings")
         self._protocol.set_compression_level(self.compression_level)
+        self._protocol.receive_aliases.update(self._aliases)
         self.have_more = self._protocol.source_has_more
 
     def init_packet_handlers(self):
@@ -143,9 +168,20 @@ class XpraClientBase(object):
             self._reverse_aliases[key] = i
             i += 1
 
-    def send_hello(self, challenge_response=None):
-        hello = self.make_hello(challenge_response)
-        log.debug("send_hello(%s) packet=%s", challenge_response, hello)
+    def send_hello(self, challenge_response=None, client_salt=None):
+        hello = self.make_hello_base()
+        if self.password_file and not challenge_response:
+            #avoid sending the full hello: tell the server we want
+            #a packet challenge first
+            hello["challenge"] = True
+        else:
+            hello.update(self.make_hello())
+        if challenge_response:
+            assert self.password_file
+            hello["challenge_response"] = challenge_response
+            if client_salt:
+                hello["challenge_client_salt"] = client_salt
+        log.debug("send_hello(%s) packet=%s", binascii.hexlify(challenge_response or ""), hello)
         self.send("hello", hello)
         self.timeout_add(DEFAULT_TIMEOUT, self.verify_connected)
 
@@ -155,68 +191,55 @@ class XpraClientBase(object):
             self.warn_and_quit(EXIT_TIMEOUT, "connection timed out")
 
 
-    def make_hello(self, challenge_response=None):
-        capabilities = {}
+    def make_hello_base(self):
+        capabilities = get_network_caps()
+        capabilities.update({
+                "encoding.generic"      : True,
+                "namespace"             : True,
+                "hostname"              : socket.gethostname(),
+                "uuid"                  : self.uuid,
+                "username"              : self.username,
+                "name"                  : get_name(),
+                "client_type"           : self.client_type(),
+                "python.version"        : sys.version_info[:3],
+                "compression_level"     : self.compression_level,
+                })
+        if self.display:
+            capabilities["display"] = self.display
+        capabilities.update(get_platform_info())
         add_version_info(capabilities)
-        capabilities["python.version"] = sys.version_info[:3]
-        if challenge_response:
-            assert self.password
-            capabilities["challenge_response"] = challenge_response
+        mid = get_machine_id()
+        if mid:
+            capabilities["machine_id"] = mid
+
         if self.encryption:
             assert self.encryption in ENCRYPTION_CIPHERS
-            capabilities["cipher"] = self.encryption
             iv = get_hex_uuid()[:16]
-            capabilities["cipher.iv"] = iv
-            key_salt = get_hex_uuid()
-            capabilities["cipher.key_salt"] = key_salt
+            key_salt = get_hex_uuid()+get_hex_uuid()
             iterations = 1000
-            capabilities["cipher.key_stretch_iterations"] = iterations
-            self._protocol.set_cipher_in(self.encryption, iv, self.get_password(), key_salt, iterations)
+            capabilities.update({
+                        "cipher"                       : self.encryption,
+                        "cipher.iv"                    : iv,
+                        "cipher.key_salt"              : key_salt,
+                        "cipher.key_stretch_iterations": iterations,
+                        })
+            key = self.get_encryption_key()
+            if key is None:
+                self.warn_and_quit(EXIT_ENCRYPTION, "encryption key is missing")
+                return
+            self._protocol.set_cipher_in(self.encryption, iv, key, key_salt, iterations)
             log("encryption capabilities: %s", [(k,v) for k,v in capabilities.items() if k.startswith("cipher")])
-        capabilities["platform"] = sys.platform
-        capabilities["platform.release"] = python_platform.release()
-        capabilities["platform.machine"] = python_platform.machine()
-        capabilities["platform.processor"] = python_platform.processor()
-        capabilities["client_type"] = self.client_type()
-        capabilities["namespace"] = True
-        capabilities["raw_packets"] = True
-        capabilities["chunked_compression"] = True
-        capabilities["bencode"] = True
-        capabilities["rencode"] = has_rencode
-        if has_rencode:
-            capabilities["rencode.version"] = rencode_version
-        capabilities["hostname"] = socket.gethostname()
-        capabilities["uuid"] = self.uuid
-        try:
-            from xpra.platform.info import get_username, get_name
-            capabilities["username"] = get_username()
-            capabilities["name"] = get_name()
-        except:
-            log.error("failed to get username/name", exc_info=True)
-        capabilities["randr_notify"] = False    #only client.py cares about this
-        capabilities["windows"] = False         #only client.py cares about this
+        return capabilities
+
+    def make_hello(self):
+        capabilities = {
+                        "randr_notify"        : False,        #only client.py cares about this
+                        "windows"            : False,        #only client.py cares about this
+                       }
         if self._reverse_aliases:
             capabilities["aliases"] = self._reverse_aliases
         return capabilities
 
-    def make_uuid(self):
-        try:
-            import hashlib
-            u = hashlib.sha1()
-        except:
-            #try python2.4 variant:
-            import sha
-            u = sha.new()
-        def uupdate(ustr):
-            u.update(ustr.encode("utf-8"))
-        uupdate(get_machine_id())
-        if os.name=="posix":
-            uupdate(u"/")
-            uupdate(str(os.getuid()))
-            uupdate(u"/")
-            uupdate(str(os.getgid()))
-        self.uuid = u.hexdigest()
-
     def send(self, *parts):
         self._ordinary_packets.append(parts)
         self.have_more()
@@ -299,25 +322,51 @@ class XpraClientBase(object):
         self.warn_and_quit(EXIT_CONNECTION_LOST, "Connection lost")
 
     def _process_challenge(self, packet):
-        if not self.password_file and not self.password:
-            self.warn_and_quit(EXIT_PASSWORD_REQUIRED, "password is required by the server")
+        log("processing challenge: %s", packet[1:])
+        if not self.password_file:
+            self.warn_and_quit(EXIT_PASSWORD_REQUIRED, "server requires authentication, please provide a password")
+            return
+        password = self.load_password()
+        if not password:
+            self.warn_and_quit(EXIT_PASSWORD_FILE_ERROR, "failed to load password from file %s" % self.password_file)
             return
-        if not self.password:
-            if not self.load_password():
-                return
-            assert self.password
         salt = packet[1]
         if self.encryption:
             assert len(packet)>=3, "challenge does not contain encryption details to use for the response"
             server_cipher = packet[2]
-            self.set_server_encryption(server_cipher)
-        import hmac
-        challenge_response = hmac.HMAC(self.password, salt)
-        password_hash = challenge_response.hexdigest()
+            key = self.get_encryption_key()
+            if key is None:
+                self.warn_and_quit(EXIT_ENCRYPTION, "encryption key is missing")
+                return
+            if not self.set_server_encryption(server_cipher, key):
+                return
+        digest = "hmac"
+        client_can_salt = len(packet)>=4
+        client_salt = None
+        if client_can_salt:
+            #server supports client salt, and tells us which digest to use:
+            digest = packet[3]
+            client_salt = get_hex_uuid()+get_hex_uuid()
+            #TODO: use some key stretching algorigthm? (meh)
+            salt = xor(salt, client_salt)
+        if digest=="hmac":
+            import hmac
+            challenge_response = hmac.HMAC(password, salt).hexdigest()
+        elif digest=="xor":
+            #don't send XORed password unencrypted:
+            if not self._protocol.cipher_out and not ALLOW_UNENCRYPTED_PASSWORDS:
+                self.warn_and_quit(EXIT_ENCRYPTION, "server requested digest %s, cowardly refusing to use it without encryption" % digest)
+                return
+            challenge_response = xor(password, salt)
+        else:
+            self.warn_and_quit(EXIT_PASSWORD_REQUIRED, "server requested an unsupported digest: %s" % digest)
+            return
+        if digest:
+            log("%s(%s, %s)=%s", digest, password, salt, challenge_response)
         self.password_sent = True
-        self.send_hello(password_hash)
+        self.send_hello(challenge_response, client_salt)
 
-    def set_server_encryption(self, capabilities):
+    def set_server_encryption(self, capabilities, key):
         def get(key, default=None):
             return capabilities.get(strtobytes(key), default)
         cipher = get("cipher")
@@ -330,33 +379,40 @@ class XpraClientBase(object):
         if cipher not in ENCRYPTION_CIPHERS:
             self.warn_and_quit(EXIT_ENCRYPTION, "unsupported server cipher: %s, allowed ciphers: %s" % (cipher, ", ".join(ENCRYPTION_CIPHERS)))
             return False
-        self._protocol.set_cipher_out(cipher, cipher_iv, self.get_password(), key_salt, iterations)
+        self._protocol.set_cipher_out(cipher, cipher_iv, key, key_salt, iterations)
+        return True
 
 
-    def get_password(self):
-        if self.password is None:
-            self.load_password()
-        return self.password
+    def get_encryption_key(self):
+        key = load_binary_file(self.encryption_keyfile)
+        if key is None and self.password_file:
+            key = load_binary_file(self.password_file)
+            if key:
+                log("used password file as encryption key")
+        if key is None:
+            raise Exception("failed to load encryption keyfile %s" % self.encryption_keyfile)
+        return key.strip("\n\r")
 
     def load_password(self):
-        try:
-            filename = os.path.expanduser(self.password_file)
-            passwordFile = open(filename, "rU")
-            self.password = passwordFile.read()
-            passwordFile.close()
-            while self.password.endswith("\n") or self.password.endswith("\r"):
-                self.password = self.password[:-1]
-        except IOError, e:
-            self.warn_and_quit(EXIT_PASSWORD_FILE_ERROR, "failed to open password file %s: %s" % (self.password_file, e))
-            return False
-        log("password read from file %s is %s", self.password_file, self.password)
-        return True
+        filename = os.path.expanduser(self.password_file)
+        password = load_binary_file(filename)
+        if password is None:
+            return None
+        password = password.strip("\n\r")
+        log("password read from file %s is %s", self.password_file, "".join(["*" for _ in password]))
+        return password
 
     def _process_hello(self, packet):
         if not self.password_sent and self.password_file:
-            log.warn("Warning: the server did not request our password!")
-        self.server_capabilities = packet[1]
-        self.parse_server_capabilities(self.server_capabilities)
+            self.warn_and_quit(EXIT_NO_AUTHENTICATION, "the server did not request our password")
+            return
+        try:
+            self.server_capabilities = packet[1]
+            log("processing hello from server: %s", self.server_capabilities)
+            c = typedict(self.server_capabilities)
+            self.parse_server_capabilities(c)
+        except Exception, e:
+            self.warn_and_quit(EXIT_FAILURE, "error processing hello packet from server: %s" % e)
 
     def capsget(self, capabilities, key, default):
         v = capabilities.get(strtobytes(key), default)
@@ -364,27 +420,32 @@ class XpraClientBase(object):
             v = bytestostr(v)
         return v
 
-    def parse_server_capabilities(self, capabilities):
-        def get(key, default=None):
-            return self.capsget(capabilities, key, default)
-        self._remote_version = get("version")
-        self._remote_revision = get("revision")
-        self._remote_revision = get("build.revision", self._remote_revision)
-        self._remote_platform = get("platform")
-        self._remote_platform_release = get("platform.release")
-        self._remote_platform_platform = get("platform.platform")
-        self._remote_platform_linux_distribution = get("platform.linux_distribution")
+    def parse_server_capabilities(self, c):
+        self._remote_machine_id = c.strget("machine_id")
+        self._remote_uuid = c.strget("uuid")
+        self._remote_version = c.strget("version")
+        self._remote_revision = c.strget("revision")
+        self._remote_revision = c.strget("build.revision", self._remote_revision)
+        self._remote_platform = c.strget("platform")
+        self._remote_platform_release = c.strget("platform.release")
+        self._remote_platform_platform = c.strget("platform.platform")
+        self._remote_platform_linux_distribution = c.get("platform.linux_distribution")
         verr = version_compat_check(self._remote_version)
         if verr is not None:
-            self.warn_and_quit(EXIT_INCOMPATIBLE_VERSION, "incompatible remote version %s: %s" % (self._remote_version, verr))
+            self.warn_and_quit(EXIT_INCOMPATIBLE_VERSION, "incompatible remote version '%s': %s" % (self._remote_version, verr))
             return False
-        if capabilities.get("rencode") and use_rencode:
+
+        if use_rencode and c.boolget("rencode"):
             self._protocol.enable_rencode()
+        if use_lz4 and c.boolget("lz4") and self.compression_level==1:
+            self._protocol.enable_lz4()
         if self.encryption:
             #server uses a new cipher after second hello:
-            self.set_server_encryption(capabilities)
-        self._protocol.chunked_compression = get("chunked_compression", False)
-        self._protocol.aliases = get("aliases", {})
+            key = self.get_encryption_key()
+            assert key, "encryption key is missing"
+            if not self.set_server_encryption(c, key):
+                return False
+        self._protocol.send_aliases = c.dictget("aliases", {})
         if self.pings:
             self.timeout_add(1000, self.send_ping)
         else:
@@ -413,8 +474,6 @@ class XpraClientBase(object):
         try:
             handler = None
             packet_type = packet[0]
-            if type(packet_type)==int:
-                packet_type = self._aliases.get(packet_type)
             handler = self._packet_handlers.get(packet_type)
             if handler:
                 handler(packet)
