@@ -1,11 +1,10 @@
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2013 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2014 Antoine Martin <antoine@devloop.org.uk>
 # Copyright (C) 2008, 2010 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-import time
 import os
 import sys
 import gobject
@@ -16,15 +15,21 @@ except:
     pass
 import gtk
 from gtk import gdk
-gtk.threads_init()
+gdk.threads_init()
 
-from xpra.os_util import thread
+
+from xpra.platform.ui_thread_watcher import get_UI_watcher
+UI_watcher = get_UI_watcher(gobject.timeout_add)
+
+from xpra.gtk_common.gtk2common import gtk2main
 from xpra.client.gtk_base.gtk_client_base import GTKXpraClient, xor_str
 from xpra.client.gtk2.tray_menu import GTK2TrayMenu
 from xpra.gtk_common.cursor_names import cursor_names
-from xpra.gtk_common.gtk2common import gtk2main
+from xpra.client.window_border import WindowBorder
 from xpra.log import Logger
-log = Logger()
+log = Logger("gtk", "client")
+cursorlog = Logger("gtk", "client", "cursor")
+clipboardlog = Logger("gtk", "client", "clipboard")
 
 from xpra.client.gtk2.border_client_window import BorderClientWindow
 from xpra.client.gtk2.client_window import ClientWindow
@@ -37,6 +42,8 @@ WINDOW_LAYOUTS = {
 
 FAKE_UI_LOCKUPS = int(os.environ.get("XPRA_FAKE_UI_LOCKUPS", "0"))
 
+missing_cursor_names = set()
+
 
 class XpraClient(GTKXpraClient):
 
@@ -45,6 +52,7 @@ class XpraClient(GTKXpraClient):
 
     def __init__(self):
         GTKXpraClient.__init__(self)
+        self.border = None
         self.GLClientWindowClass = None
         self.local_clipboard_requests = 0
         self.remote_clipboard_requests = 0
@@ -52,6 +60,7 @@ class XpraClient(GTKXpraClient):
         #avoid ugly "not implemented" warning on win32
         self.supports_group_leader = not sys.platform.startswith("win")
 
+        self.window_with_grab = None
         self._ref_to_group_leader = {}
         self._group_leader_wids = {}
 
@@ -61,12 +70,53 @@ class XpraClient(GTKXpraClient):
             assert opts.window_layout in WINDOW_LAYOUTS
             self.ClientWindowClass = WINDOW_LAYOUTS.get(opts.window_layout)
         else:
-            self.ClientWindowClass = ClientWindow
+            self.ClientWindowClass = BorderClientWindow
         log("init(..) ClientWindowClass=%s", self.ClientWindowClass)
 
+
+    def parse_border(self, border_str):
+        parts = [x.strip() for x in border_str.split(",")]
+        color_str = parts[0]
+        if color_str.lower()=="none":
+            return
+        if color_str=="auto":
+            try:
+                import hashlib
+                m = hashlib.sha1()
+                m.update(self._protocol._conn.target)
+                color_str = "#%s" % m.hexdigest()[:6]
+                log("border color derived from %s: %s", self._protocol._conn.target, color_str)
+            except:
+                #fail: default to red
+                color_str = "red"
+        try:
+            color = gtk.gdk.color_parse(color_str)
+        except Exception, e:
+            log.warn("invalid color specified: %s (%s)", color_str, e)
+            return
+        alpha = 0.6
+        size = 4
+        if len(parts)==2:
+            size_str = parts[1]
+            try:
+                size = int(size_str)
+            except Exception, e:
+                log.warn("invalid size specified: %s (%s)", size_str, e)
+            if size<=0:
+                log("border size is %s, disabling it", size)
+                return
+        self.border = WindowBorder(True, color.red_float, color.green_float, color.blue_float, alpha, size)
+        log("parse_border(%s)=%s", border_str, self.border)
+
+
     def gtk_main(self):
         gtk2main()
 
+    def cleanup(self):
+        global UI_watcher
+        UI_watcher.stop()
+        GTKXpraClient.cleanup(self)
+
     def client_type(self):
         return "Python/Gtk2"
 
@@ -79,25 +129,22 @@ class XpraClient(GTKXpraClient):
             from xpra.client.gtk2.gtk2_notifier import GTK2_Notifier
             ncs.append(GTK2_Notifier)
         except Exception, e:
-            log("cannot load GTK3 notifier: %s", e)
+            log.warn("cannot load GTK2 notifier: %s", e)
         return ncs
 
+    def make_new_window(self, *args):
+        w = GTKXpraClient.make_new_window(self, *args)
+        if w:
+            w.border = self.border
+        return w
 
     def get_supported_window_layouts(self):
         return  WINDOW_LAYOUTS
 
-    def get_core_encodings(self):
-        encodings = GTKXpraClient.get_core_encodings(self)
-        if gdk.screen_get_default().get_rgba_visual() is not None:
-            try:
-                #check for bytearray which is used by PixmapBacking
-                #to unpremultiply rgb32 data
-                bytearray("")
-                from xpra.codecs.argb.argb import unpremultiply_argb_in_place
-                assert unpremultiply_argb_in_place
-                encodings.append("rgb32")
-            except:
-                pass
+    def do_get_core_encodings(self):
+        encodings = GTKXpraClient.do_get_core_encodings(self)
+        #we can handle rgb32 format (but not necessarily transparency!)
+        encodings.append("rgb32")
         #gtk2 can handle 'png' and 'jpeg' natively (without PIL)
         #(though using PIL is better since we can do that in the decode thread)
         for x in ("png", "jpeg"):
@@ -133,6 +180,17 @@ class XpraClient(GTKXpraClient):
         return tmhc
 
 
+    def process_clipboard_packet(self, packet):
+        clipboardlog("process_clipboard_packet(%s) level=%s", packet, gtk.main_level())
+        #check for clipboard loops:
+        if gtk.main_level()>=10:
+            clipboardlog.warn("loop nesting too deep: %s", gtk.main_level())
+            clipboardlog.warn("you may have a clipboard forwarding loop, disabling the clipboard")
+            self.clipboard_enabled = False
+            self.emit("clipboard-toggled")
+            return
+        self.idle_add(self.clipboard_helper.process_clipboard_packet, packet)
+
     def make_clipboard_helper(self):
         """
             Try the various clipboard classes until we find one
@@ -140,14 +198,14 @@ class XpraClient(GTKXpraClient):
         """
         from xpra.platform.features import CLIPBOARDS, CLIPBOARD_NATIVE_CLASS
         clipboards = [x for x in CLIPBOARDS if x in self.server_clipboards]
-        log("make_clipboard_helper() server_clipboards=%s, local clipboards=%s, common=%s", self.server_clipboards, CLIPBOARDS, clipboards)
+        clipboardlog("make_clipboard_helper() server_clipboards=%s, local clipboards=%s, common=%s", self.server_clipboards, CLIPBOARDS, clipboards)
         #first add the platform specific one, (may be None):
         clipboard_options = []
         if CLIPBOARD_NATIVE_CLASS:
             clipboard_options.append(CLIPBOARD_NATIVE_CLASS)
         clipboard_options.append(("xpra.clipboard.gdk_clipboard", "GDKClipboardProtocolHelper", {"clipboards" : clipboards}))
         clipboard_options.append(("xpra.clipboard.clipboard_base", "DefaultClipboardProtocolHelper", {"clipboards" : clipboards}))
-        log("make_clipboard_helper() clipboard_options=%s", clipboard_options)
+        clipboardlog("make_clipboard_helper() clipboard_options=%s", clipboard_options)
         for module_name, classname, kwargs in clipboard_options:
             c = self.try_load_clipboard_helper(module_name, classname, kwargs)
             if c:
@@ -165,20 +223,20 @@ class XpraClient(GTKXpraClient):
                 if c:
                     return self.setup_clipboard_helper(c, **kwargs)
         except:
-            log.error("cannot load %s.%s", module, classname, exc_info=True)
+            clipboardlog.error("cannot load %s.%s", module, classname, exc_info=True)
             return None
-        log.error("cannot load %s.%s", module, classname)
+        clipboardlog.error("cannot load %s.%s", module, classname)
         return None
 
     def setup_clipboard_helper(self, helperClass, *args, **kwargs):
-        log("setup_clipboard_helper(%s, %s, %s)", helperClass, args, kwargs)
+        clipboardlog("setup_clipboard_helper(%s, %s, %s)", helperClass, args, kwargs)
         def clipboard_send(*parts):
             if self.clipboard_enabled:
                 self.send(*parts)
             else:
-                log("clipboard is disabled, not sending clipboard packet")
+                clipboardlog("clipboard is disabled, not sending clipboard packet")
         def clipboard_progress(local_requests, remote_requests):
-            log("clipboard_progress(%s, %s)", local_requests, remote_requests)
+            clipboardlog("clipboard_progress(%s, %s)", local_requests, remote_requests)
             if local_requests is not None:
                 self.local_clipboard_requests = local_requests
             if remote_requests is not None:
@@ -187,7 +245,7 @@ class XpraClient(GTKXpraClient):
             self.clipboard_notify(n)
         def register_clipboard_toggled(*args):
             def clipboard_toggled(*targs):
-                log("clipboard_toggled(%s) enabled=%s, server_supports_clipboard=%s", targs, self.clipboard_enabled, self.server_supports_clipboard)
+                clipboardlog("clipboard_toggled(%s) enabled=%s, server_supports_clipboard=%s", targs, self.clipboard_enabled, self.server_supports_clipboard)
                 if self.clipboard_enabled and self.server_supports_clipboard:
                     assert self.clipboard_helper is not None
                     self.clipboard_helper.send_all_tokens()
@@ -204,15 +262,30 @@ class XpraClient(GTKXpraClient):
     def clipboard_notify(self, n):
         if not self.tray:
             return
-        self.tray.set_blinking(n>0 and self.clipboard_enabled)
+        clipboardlog("clipboard_notify(%s)", n)
+        if n>0 and self.clipboard_enabled:
+            self.tray.set_icon("clipboard")
+            self.tray.set_tooltip("%s clipboard requests in progress" % n)
+            self.tray.set_blinking(True)
+        else:
+            self.tray.set_icon(None)    #None means back to default icon
+            self.tray.set_tooltip(self.get_tray_title())
+            self.tray.set_blinking(False)
 
 
-    def make_hello(self, challenge_response=None):
-        capabilities = GTKXpraClient.make_hello(self, challenge_response)
+    def make_hello(self):
+        capabilities = GTKXpraClient.make_hello(self)
         if xor_str is not None:
             capabilities["encoding.supports_delta"] = [x for x in ("png", "rgb24", "rgb32") if x in self.get_core_encodings()]
+        capabilities["pointer.grabs"] = True
         return capabilities
 
+    def init_packet_handlers(self):
+        GTKXpraClient.init_packet_handlers(self)
+        self._ui_packet_handlers["pointer-grab"] = self._process_pointer_grab
+        self._ui_packet_handlers["pointer-ungrab"] = self._process_pointer_ungrab
+
+
     def process_ui_capabilities(self, capabilities):
         GTKXpraClient.process_ui_capabilities(self, capabilities)
         if self.server_randr:
@@ -220,61 +293,19 @@ class XpraClient(GTKXpraClient):
             i=0
             while i<display.get_n_screens():
                 screen = display.get_screen(i)
-                screen.connect("size-changed", self._screen_size_changed)
+                screen.connect("size-changed", self.screen_size_changed)
                 i += 1
+        global UI_watcher
+        UI_watcher.start()
         #if server supports it, enable UI thread monitoring workaround when needed:
-        if self.suspend_resume and (sys.platform.startswith("darwin") or FAKE_UI_LOCKUPS>0):
-            self.start_UI_thread_polling()
-            if FAKE_UI_LOCKUPS>0:
-                def sleep_in_ui_thread(*args):
-                    time.sleep(FAKE_UI_LOCKUPS)
-                    return True
-                gobject.timeout_add((10+FAKE_UI_LOCKUPS)*1000, sleep_in_ui_thread)
-
-    def start_UI_thread_polling(self):
-        log("start_UI_thread_polling()")
-        self.UI_blocked_sent = False
-        def UI_thread_wakeup(*args):
-            log("UI_thread_wakeup()")
-            self.last_UI_thread_time = time.time()
-            #UI thread was blocked?
-            if self.UI_blocked_sent:
-                log.info("UI thread is running again, resuming")
+        if self.suspend_resume:
+            def UI_resumed():
                 self.send("resume", True, self._id_to_window.keys())
-                self.UI_blocked_sent = False
-            return self.exit_code is None
-        UI_thread_wakeup()
-        gobject.timeout_add(1000, UI_thread_wakeup)
-        def poll_UI_loop(*args):
-            log("poll_UI_loop() running")
-            while self.exit_code is None:
-                delta = time.time()-self.last_UI_thread_time
-                log("poll_UI_loop() last_UI_thread_time was %.1f seconds ago, UI_blocked_sent=%s", delta, self.UI_blocked_sent)
-                if delta>2.0:
-                    #UI thread is (still?) blocked:
-                    if not self.UI_blocked_sent:
-                        log.info("UI thread is blocked, pausing server")
-                        self.send("suspend", True, self._id_to_window.keys())
-                        self.UI_blocked_sent = True
-                time.sleep(1.0)
-        thread.start_new_thread(poll_UI_loop, ())
+            def UI_failed():
+                self.send("suspend", True, self._id_to_window.keys())
+            UI_watcher.add_resume_callback(UI_resumed)
+            UI_watcher.add_fail_callback(UI_failed)
 
-    def _screen_size_changed(self, *args):
-        def update_size(current=None):
-            root_w, root_h = self.get_root_size()
-            ss = self.get_screen_sizes()
-            log("update_size(%s) sizes=%s", current, ss)
-            if current is not None and current==ss:
-                #unchanged
-                return
-            log.info("sending updated screen size to server: %sx%s, screen sizes: %s", root_w, root_h, ss)
-            self.send("desktop_size", root_w, root_h, ss)
-            #update the max packet size (may have gone up):
-            self.set_max_packet_size()
-            #check again soon:
-            gobject.timeout_add(1000, update_size, ss)
-        #update via idle_add so the data is actually up to date when we query it!
-        self.idle_add(update_size)
 
     def get_screen_sizes(self):
         display = gdk.display_get_default()
@@ -298,12 +329,12 @@ class XpraClient(GTKXpraClient):
                 monitor = plug_name, geom.x, geom.y, geom.width, geom.height, wmm, hmm
                 monitors.append(monitor)
                 j += 1
-            root = screen.get_root_window()
             work_x, work_y = 0, 0
             work_width, work_height = screen.get_width(), screen.get_height()
             if not sys.platform.startswith("win"):
                 try:
                     p = gtk.gdk.atom_intern('_NET_WORKAREA')
+                    root = screen.get_root_window()
                     work_x, work_y, work_width, work_height = root.property_get(p)[2][:4]
                 except:
                     pass
@@ -313,41 +344,62 @@ class XpraClient(GTKXpraClient):
                         work_x, work_y, work_width, work_height)
             screen_sizes.append(item)
             i += 1
-        log("get_screen_sizes()=%s", screen_sizes)
         return screen_sizes
 
 
     def get_root_size(self):
         return gdk.get_default_root_window().get_size()
 
-    def set_windows_cursor(self, gtkwindows, new_cursor):
+    def make_cursor(self, cursor_data):
+        #if present, try cursor ny name:
+        if len(cursor_data)>=9 and cursor_names:
+            cursor_name = cursor_data[8]
+            if cursor_name:
+                gdk_cursor = cursor_names.get(cursor_name.upper())
+                if gdk_cursor is not None:
+                    cursorlog("setting new cursor by name: %s=%s", cursor_name, gdk_cursor)
+                    return gdk.Cursor(gdk_cursor)
+                else:
+                    global missing_cursor_names
+                    if cursor_name not in missing_cursor_names:
+                        cursorlog("cursor name '%s' not found", cursor_name)
+                        missing_cursor_names.add(cursor_name)
+        #create cursor from the pixel data:
+        w, h, xhot, yhot, serial, pixels = cursor_data[2:8]
+        if len(pixels)<w*h*4:
+            import binascii
+            cursorlog.warn("not enough pixels provided in cursor data: %s needed and only %s bytes found (%s)", w*h*4, len(pixels), binascii.hexlify(pixels)[:100])
+            return
+        pixbuf = gdk.pixbuf_new_from_data(pixels, gdk.COLORSPACE_RGB, True, 8, w, h, w * 4)
+        x = max(0, min(xhot, w-1))
+        y = max(0, min(yhot, h-1))
+        display = gdk.display_get_default()
+        csize = display.get_default_cursor_size()
+        cmaxw, cmaxh = display.get_maximal_cursor_size()
+        if len(cursor_data)>=11:
+            ssize = cursor_data[9]
+            smax = cursor_data[10]
+            cursorlog("server cursor sizes: default=%s, max=%s", ssize, smax)
+        cursorlog("new cursor at %s,%s with serial=%s, dimensions: %sx%s, len(pixels)=%s, default cursor size is %s, maximum=%s", xhot,yhot, serial, w,h, len(pixels), csize, (cmaxw, cmaxh))
+        ratio = 1
+        if w>cmaxw or h>cmaxh or (csize>0 and (csize<w or csize<h)):
+            ratio = max(float(w)/cmaxw, float(h)/cmaxh, float(max(w,h))/csize)
+            cursorlog("downscaling cursor by %.2f", ratio)
+            pixbuf = pixbuf.scale_simple(int(w/ratio), int(h/ratio), gdk.INTERP_BILINEAR)
+            x = int(x/ratio)
+            y = int(y/ratio)
+        return gdk.Cursor(gdk.display_get_default(), pixbuf, x, y)
+
+    def set_windows_cursor(self, gtkwindows, cursor_data):
         cursor = None
-        if len(new_cursor)>0:
-            cursor = None
-            if len(new_cursor)>=9 and cursor_names:
-                cursor_name = new_cursor[8]
-                if cursor_name:
-                    gdk_cursor = cursor_names.get(cursor_name.upper())
-                    if gdk_cursor is not None:
-                        try:
-                            from xpra.x11.gtk_x11.error import trap
-                            log("setting new cursor: %s=%s", cursor_name, gdk_cursor)
-                            cursor = trap.call_synced(gdk.Cursor, gdk_cursor)
-                        except:
-                            pass
+        if cursor_data:
+            try:
+                cursor = self.make_cursor(cursor_data)
+            except Exception, e:
+                cursorlog.warn("error creating cursor: %s (using default)", e, exc_info=True)
             if cursor is None:
-                w, h, xhot, yhot, serial, pixels = new_cursor[2:8]
-                log("new cursor at %s,%s with serial=%s, dimensions: %sx%s, len(pixels)=%s" % (xhot,yhot, serial, w,h, len(pixels)))
-                pixbuf = gdk.pixbuf_new_from_data(pixels, gdk.COLORSPACE_RGB, True, 8, w, h, w * 4)
-                x = max(0, min(xhot, w-1))
-                y = max(0, min(yhot, h-1))
-                size = gdk.display_get_default().get_default_cursor_size() 
-                if size>0 and (size<w or size<h):
-                    ratio = float(max(w,h))/size
-                    pixbuf = pixbuf.scale_simple(int(w/ratio), int(h/ratio), gdk.INTERP_BILINEAR)
-                    x = int(x/ratio)
-                    y = int(y/ratio)
-                cursor = gdk.Cursor(gdk.display_get_default(), pixbuf, x, y)
+                #use default:
+                cursor = gdk.Cursor(gtk.gdk.X_CURSOR)
         for gtkwindow in gtkwindows:
             if gtk.gtk_version>=(2,14):
                 gdkwin = gtkwindow.get_window()
@@ -358,6 +410,37 @@ class XpraClient(GTKXpraClient):
                 gdkwin.set_cursor(cursor)
 
 
+    def _process_raise_window(self, packet):
+        wid = packet[1]
+        window = self._id_to_window.get(wid)
+        log("going to raise window %s - %s", wid, window)
+        if window:
+            if window.has_toplevel_focus():
+                log("window already has top level focus")
+                return
+            window.present()
+
+
+    def _process_pointer_grab(self, packet):
+        wid = packet[1]
+        window = self._id_to_window.get(wid)
+        if window:
+            log("grabbing %s", window)
+            mask = gtk.gdk.BUTTON_PRESS_MASK | gtk.gdk.BUTTON_RELEASE_MASK | gtk.gdk.POINTER_MOTION_MASK  | gtk.gdk.POINTER_MOTION_HINT_MASK | gtk.gdk.ENTER_NOTIFY_MASK | gtk.gdk.LEAVE_NOTIFY_MASK
+            gtk.gdk.pointer_grab(window.gdk_window(), owner_events=True, event_mask=mask)
+            #also grab the keyboard so the user won't Alt-Tab away:
+            gtk.gdk.keyboard_grab(window.gdk_window(), owner_events=False)
+            self.window_with_grab = wid
+
+    def _process_pointer_ungrab(self, packet):
+        wid = packet[1]
+        window = self._id_to_window.get(wid)
+        log("ungrabbing %s", window)
+        gtk.gdk.pointer_ungrab()
+        gtk.gdk.keyboard_ungrab()
+        self.window_with_grab = None
+
+
     def init_opengl(self, enable_opengl):
         #enable_opengl can be True, False or None (auto-detect)
         self.client_supports_opengl = False
@@ -368,7 +451,7 @@ class XpraClient(GTKXpraClient):
         check = OpenGL_safety_check()
         if check:
             if enable_opengl is True:
-                log.warn("OpenGL enabled despite: %s", check)
+                log.warn("OpenGL loading despite: %s", check)
             else:
                 self.opengl_props["info"] = "disabled: %s" % check
                 log.warn("OpenGL disabled: %s", check)
@@ -440,7 +523,13 @@ class XpraClient(GTKXpraClient):
         return group_leader_window
 
     def destroy_window(self, wid, window):
-        #override so we can cleanup the group-leader if needed:
+        #override so we can cleanup the group-leader if needed,
+        #and lose the grab
+        if self.window_with_grab==wid:
+            log("destroying window %s which has grab, ungrabbing!", wid)
+            gtk.gdk.pointer_ungrab()
+            gtk.gdk.keyboard_ungrab()
+            self.window_with_grab = None
         GTKXpraClient.destroy_window(self, wid, window)
         group_leader = window.group_leader
         if group_leader is None or len(self._group_leader_wids)==0:
@@ -467,20 +556,13 @@ class XpraClient(GTKXpraClient):
         group_leader.destroy()
 
 
-    def get_client_window_class(self, metadata, override_redirect):
+    def get_client_window_classes(self, metadata, override_redirect):
         #only enable GL for normal windows:
         window_types = metadata.get("window-type", ())
         log("get_client_window_class(%s, %s) GLClientWindowClass=%s, opengl_enabled=%s, mmap_enabled=%s, window_types=%s, encoding=%s", metadata, override_redirect, self.GLClientWindowClass, self.opengl_enabled, self.mmap_enabled, window_types, self.encoding)
-        if self.GLClientWindowClass is None or not self.opengl_enabled or override_redirect:
-            return self.ClientWindowClass
-        if metadata.get("has-alpha", False):
-            #GL cannot do transparency yet:
-            return self.ClientWindowClass
-        if self.mmap_enabled or self.encoding not in ("x264", "vpx"):
-            return self.ClientWindowClass
-        if ("NORMAL" not in window_types) and ("_NET_WM_WINDOW_TYPE_NORMAL" not in window_types):
-            return self.ClientWindowClass
-        return self.GLClientWindowClass
+        if self.GLClientWindowClass is None or not self.opengl_enabled:
+            return [self.ClientWindowClass]
+        return [self.GLClientWindowClass, self.ClientWindowClass]
 
     def toggle_opengl(self, *args):
         assert self.window_unmap, "server support for 'window_unmap' is required for toggling opengl at runtime"
