@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2013 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2013, 2014 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -14,7 +14,7 @@ from gi.repository.GdkPixbuf import InterpType  #@UnresolvedImport
 from xpra.client.gtk3.client_window import ClientWindow
 from xpra.client.gtk3.tray_menu import GTK3TrayMenu
 from xpra.log import Logger
-log = Logger()
+log = Logger("gtk", "client")
 
 
 class XpraClient(GTKXpraClient):
@@ -26,8 +26,8 @@ class XpraClient(GTKXpraClient):
     def gtk_main(self):
         Gtk.main()
 
-    def make_hello(self, challenge_response=None):
-        capabilities = GTKXpraClient.make_hello(self, challenge_response)
+    def make_hello(self):
+        capabilities = GTKXpraClient.make_hello(self)
         if xor_str is not None:
             capabilities["encoding.supports_delta"] = [x for x in ("rgb24", "rgb32") if x in self.get_core_encodings()]
         return capabilities
