@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2013 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2014 Antoine Martin <antoine@devloop.org.uk>
 # Copyright (C) 2008, 2010 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -14,7 +14,7 @@ gdk = import_gdk()
 
 
 from xpra.log import Logger
-log = Logger()
+log = Logger("gtk", "main")
 
 from xpra.gtk_common.quit import (gtk_main_quit_really,
                            gtk_main_quit_on_fatal_exceptions_enable)
@@ -23,6 +23,7 @@ from xpra.gtk_common.gtk_util import add_gtk_version_info
 from xpra.client.ui_client_base import UIXpraClient
 from xpra.client.gobject_client_base import GObjectXpraClient
 from xpra.client.gtk_base.gtk_keyboard_helper import GTKKeyboardHelper
+from xpra.client.gtk_base.session_info import SessionInfo
 from xpra.platform.paths import get_icon_filename
 from xpra.platform.gui import system_bell
 try:
@@ -40,6 +41,7 @@ class GTKXpraClient(UIXpraClient, GObjectXpraClient):
     def __init__(self):
         GObjectXpraClient.__init__(self)
         UIXpraClient.__init__(self)
+        self.session_info = None
 
     def init(self, opts):
         GObjectXpraClient.init(self, opts)
@@ -69,6 +71,24 @@ class GTKXpraClient(UIXpraClient, GObjectXpraClient):
             log("GTKXpraClient.quit(%s) main loop at level %s, calling gtk quit via timeout", exit_code, gtk.main_level())
             gobject.timeout_add(500, gtk_main_quit_really)
 
+    def cleanup(self):
+        if self.session_info:
+            self.session_info.destroy()
+        UIXpraClient.cleanup(self)
+
+    def show_session_info(self, *args):
+        if self.session_info and not self.session_info.is_closed:
+            #exists already: just raise its window:
+            self.session_info.set_args(*args)
+            self.session_info.present()
+            return
+        pixbuf = self.get_pixbuf("statistics.png")
+        if not pixbuf:
+            pixbuf = self.get_pixbuf("xpra.png")
+        self.session_info = SessionInfo(self, self.session_name, pixbuf, self._protocol._conn, self.get_pixbuf)
+        self.session_info.set_args(*args)
+        self.session_info.show_all()
+
 
     def get_pixbuf(self, icon_name):
         try:
@@ -139,8 +159,8 @@ class GTKXpraClient(UIXpraClient, GObjectXpraClient):
         return self.mask_to_names(modifiers_mask)
 
 
-    def make_hello(self, challenge_response=None):
-        capabilities = UIXpraClient.make_hello(self, challenge_response)
+    def make_hello(self):
+        capabilities = UIXpraClient.make_hello(self)
         capabilities["named_cursors"] = len(cursor_names)>0
         add_gtk_version_info(capabilities, gtk, "", True)
         return capabilities
