@@ -5,10 +5,15 @@
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
-from xpra.client.client_window_base import ClientWindowBase
+from xpra.log import Logger
+focuslog = Logger("focus")
+workspacelog = Logger("workspace")
+log = Logger("window")
+keylog = Logger("window", "keyboard")
 
 from xpra.util import AdHocStruct, nn
 from xpra.gtk_common.gobject_compat import import_gtk, import_gdk
+from xpra.client.client_window_base import ClientWindowBase
 gtk = import_gtk()
 gdk = import_gdk()
 
@@ -23,30 +28,48 @@ HAS_X11_BINDINGS = False
 if os.name=="posix":
     try:
         from xpra.x11.gtk_x11.prop import prop_get, prop_set
-        from xpra.x11.gtk_x11.gdk_bindings import get_xwindow
-        from xpra.x11.bindings.window_bindings import const, X11WindowBindings  #@UnresolvedImport
+        from xpra.x11.bindings.window_bindings import constants, X11WindowBindings  #@UnresolvedImport
         from xpra.x11.gtk_x11.error import trap
+        X11Window = X11WindowBindings()
         HAS_X11_BINDINGS = True
+
+        SubstructureNotifyMask = constants["SubstructureNotifyMask"]
+        SubstructureRedirectMask = constants["SubstructureRedirectMask"]
+        CurrentTime = constants["CurrentTime"]
+
         try:
             #TODO: in theory this is not a proper check, meh - that will do
             root = gtk.gdk.get_default_root_window()
             supported = prop_get(root, "_NET_SUPPORTED", ["atom"], ignore_errors=True)
             CAN_SET_WORKSPACE = bool(supported) and "_NET_WM_DESKTOP" in supported
-        except:
-            pass
+        except Exception, e:
+            log.info("failed to setup workspace hooks: %s", e)
     except ImportError, e:
         pass
 
 
+#optional module providing faster handling of premultiplied argb:
+try:
+    from xpra.codecs.argb.argb import unpremultiply_argb, byte_buffer_to_buffer   #@UnresolvedImport
+except:
+    unpremultiply_argb, byte_buffer_to_buffer  = None, None
+
+
 class GTKKeyEvent(AdHocStruct):
     pass
 
+
 class GTKClientWindowBase(ClientWindowBase, gtk.Window):
 
     def init_window(self, metadata):
         self._fullscreen = None
-        self._can_set_workspace = HAS_X11_BINDINGS and CAN_SET_WORKSPACE
+        self._iconified = False
+        self._resize_counter = 0
+        self._window_workspace = self._client_properties.get("workspace")
+        self._desktop_workspace = -1
         ClientWindowBase.init_window(self, metadata)
+        self._can_set_workspace = HAS_X11_BINDINGS and CAN_SET_WORKSPACE
+
 
     def setup_window(self):
         ClientWindowBase.setup_window(self)
@@ -60,9 +83,9 @@ class GTKClientWindowBase(ClientWindowBase, gtk.Window):
         if not self._override_redirect:
             self.connect("notify::has-toplevel-focus", self._focus_change)
         def focus_in(*args):
-            self.debug("focus-in-event for wid=%s", self._id)
+            focuslog("focus-in-event for wid=%s", self._id)
         def focus_out(*args):
-            self.debug("focus-out-event for wid=%s", self._id)
+            focuslog("focus-out-event for wid=%s", self._id)
         self.connect("focus-in-event", focus_in)
         self.connect("focus-out-event", focus_out)
         if self._can_set_workspace:
@@ -83,13 +106,28 @@ class GTKClientWindowBase(ClientWindowBase, gtk.Window):
     def window_state_updated(self, widget, event):
         self._fullscreen = bool(event.new_window_state & self.WINDOW_STATE_FULLSCREEN)
         maximized = bool(event.new_window_state & self.WINDOW_STATE_MAXIMIZED)
+        iconified = bool(event.new_window_state & self.WINDOW_STATE_ICONIFIED)
         self._client_properties["maximized"] = maximized
-        self.debug("window_state_updated(%s, %s) new_window_state=%s, fullscreen=%s, maximized=%s", widget, repr(event), event.new_window_state, self._fullscreen, maximized)
+        log("%s.window_state_updated(%s, %s) new_window_state=%s, fullscreen=%s, maximized=%s, iconified=%s", self, widget, repr(event), event.new_window_state, self._fullscreen, maximized, iconified)
+        if iconified!=self._iconified:
+            #handle iconification as map events:
+            assert not self._override_redirect
+            if iconified:
+                assert not self._iconified
+                #usually means it is unmapped
+                self._iconified = True
+                self._unfocus()
+                if not self._override_redirect:
+                    self.send("unmap-window", self._id, True)
+            else:
+                assert not iconified and self._iconified
+                self._iconified = False
+                self.process_map_event()
 
     def set_fullscreen(self, fullscreen):
         if self._fullscreen is None or self._fullscreen!=fullscreen:
             #note: the "_fullscreen" flag is updated by the window-state-event, not here
-            self.debug("set_fullscreen(%s)", fullscreen)
+            log("%s.set_fullscreen(%s)", self, fullscreen)
             if fullscreen:
                 self.fullscreen()
             else:
@@ -103,12 +141,12 @@ class GTKClientWindowBase(ClientWindowBase, gtk.Window):
                 iid = int(xid, 16)
                 self.xset_u32_property(self.gdk_window(), "XID", iid)
             except Exception, e:
-                self.debug("set_xid(%s) error parsing/setting xid: %s", xid, e)
+                log("%s.set_xid(%s) error parsing/setting xid: %s", self, xid, e)
                 return
 
     def xget_u32_property(self, target, name):
         v = prop_get(target, name, "u32", ignore_errors=True)
-        self.debug("xget_u32_property(%s, %s)=%s", target, name, v)
+        log("%s.xget_u32_property(%s, %s)=%s", self, target, name, v)
         if type(v)==int:
             return  v
         return None
@@ -125,31 +163,67 @@ class GTKClientWindowBase(ClientWindowBase, gtk.Window):
 
 
     def property_changed(self, widget, event):
-        self.debug("property_changed: %s", event.atom)
+        log("%s.property_changed(%s, %s) : %s", self, widget, event, event.atom)
         if event.atom=="_NET_WM_DESKTOP" and self._been_mapped and not self._override_redirect:
-            #fake a configure event to send the new client_properties with
-            #the updated workspace number:
-            self.process_configure_event()
+            self.do_workspace_changed(event)
 
-    def do_set_workspace(self, workspace):
-        assert HAS_X11_BINDINGS
+    def workspace_changed(self):
+        #on X11 clients, this fires from the root window property watcher 
+        ClientWindowBase.workspace_changed(self)
+        self.do_workspace_changed("desktop workspace changed")
+    
+    def do_workspace_changed(self, info):
+        #call this method whenever something workspace related may have changed
+        window_workspace = self.get_window_workspace()
+        desktop_workspace = self.get_desktop_workspace()
+        workspacelog("do_worskpace_changed(%s) window/desktop: from %s to %s", info, (self._window_workspace, self._desktop_workspace), (window_workspace, desktop_workspace))
+        if self._window_workspace==window_workspace and self._desktop_workspace==desktop_workspace:
+            #no change
+            return
+        if desktop_workspace<0 or window_workspace<0:
+            #not sure about which workspace we are on!?
+            #maybe the property has been cleared? maybe the window is being scrubbed? do nothing.
+            return
+        if not self._client.window_refresh_config:
+            workspacelog("sending configure event to update workspace value")
+            self.process_configure_event()
+            return
+        #we can tell the server using a "buffer-refresh" packet instead
+        #and also take care of tweaking the batch config
+        client_properties = {"workspace" : window_workspace}
+        suspend_resume = None
+        options = {"refresh-now" : False}               #no need to refresh it
+        if desktop_workspace!=window_workspace:
+            workspacelog("window is on a different workspace, increasing its batch delay")
+            suspend_resume = True
+        elif self._window_workspace!=self._desktop_workspace:
+            workspacelog("window was on a different workspace, resetting its batch delay")
+            suspend_resume = False
+        self._client.control_refresh(self._id, suspend_resume, refresh=False, options=options, client_properties=client_properties)
+        self._window_workspace = window_workspace
+        self._desktop_workspace = desktop_workspace
+
+
+    def set_workspace(self):
+        if not HAS_X11_BINDINGS or not self._been_mapped:
+            return -1
         root = self.gdk_window().get_screen().get_root_window()
         ndesktops = self.xget_u32_property(root, "_NET_NUMBER_OF_DESKTOPS")
-        self.debug("set_workspace() ndesktops=%s", ndesktops)
+        workspacelog("%s.set_workspace() workspace=%s ndesktops=%s", self, self._window_workspace, ndesktops)
         if ndesktops is None or ndesktops<=1:
             return  -1
-        workspace = max(0, min(ndesktops-1, workspace))
-        event_mask = const["SubstructureNotifyMask"] | const["SubstructureRedirectMask"]
+        workspace = max(0, min(ndesktops-1, self._window_workspace))
+        event_mask = SubstructureNotifyMask | SubstructureRedirectMask
+
         def send():
-            root_window = get_xwindow(root)
-            window = get_xwindow(self.gdk_window())
-            X11WindowBindings.sendClientMessage(root_window, window, False, event_mask, "_NET_WM_DESKTOP",
-                  workspace, const["CurrentTime"],
-                  0, 0, 0)
+            root_xid = root.xid
+            xwin = self.gdk_window().xid
+            X11Window.sendClientMessage(root_xid, xwin, False, event_mask, "_NET_WM_DESKTOP",
+                  workspace, CurrentTime, 0, 0, 0)
         trap.call_synced(send)
         return workspace
 
-    def get_current_workspace(self):
+    def get_desktop_workspace(self):
         return -1
 
     def get_window_workspace(self):
@@ -160,30 +234,34 @@ class GTKClientWindowBase(ClientWindowBase, gtk.Window):
         if wid==-1:
             #root is a gdk window, so we need to ensure we have one
             #backing our gtk window to be able to call set_transient_for on it
-            self.debug("apply_transient_for(%s) gdkwindow=%s, mapped=%s", wid, self.gdk_window(), self.is_mapped())
+            log("%s.apply_transient_for(%s) gdkwindow=%s, mapped=%s", self, wid, self.gdk_window(), self.is_mapped())
             if self.gdk_window() is None:
                 self.realize()
             self.gdk_window().set_transient_for(gtk.gdk.get_default_root_window())
         else:
             #gtk window is easier:
             window = self._client._id_to_window.get(wid)
-            self.debug("apply_transient_for(%s) window=%s", wid, window)
+            log("%s.apply_transient_for(%s) window=%s", self, wid, window)
             if window:
                 self.set_transient_for(window)
 
     def update_icon(self, width, height, coding, data):
-        self.debug("update_icon(%s, %s, %s, %s bytes)", width, height, coding, len(data))
+        log("%s.update_icon(%s, %s, %s, %s bytes)", self, width, height, coding, len(data))
         if coding == "premult_argb32":
-            cairo_surf = cairo.ImageSurface(cairo.FORMAT_ARGB32, width, height)
-            cairo_surf.get_data()[:] = data
-            # FIXME: We round-trip through PNG. This is ridiculous, but faster
-            # than doing a bunch of alpha un-premultiplying and byte-swapping
-            # by hand in Python (better still would be to write some Pyrex,
-            # but I don't have time right now):
-            loader = gdk.PixbufLoader()
-            cairo_surf.write_to_png(loader)
-            loader.close()
-            pixbuf = loader.get_pixbuf()
+            if unpremultiply_argb is not None:
+                #we usually cannot do in-place and this is not performance critical
+                data = byte_buffer_to_buffer(unpremultiply_argb(data))
+                pixbuf = gdk.pixbuf_new_from_data(data, gtk.gdk.COLORSPACE_RGB, True, 8, width, height, width*4)
+            else:
+                # slower fallback: we round-trip through PNG.
+                # This is ridiculous, but faster than doing a bunch of alpha
+                # un-premultiplying and byte-swapping by hand in Python
+                cairo_surf = cairo.ImageSurface(cairo.FORMAT_ARGB32, width, height)
+                cairo_surf.get_data()[:] = data
+                loader = gdk.PixbufLoader()
+                cairo_surf.write_to_png(loader)
+                loader.close()
+                pixbuf = loader.get_pixbuf()
         else:
             loader = gdk.PixbufLoader(coding)
             loader.write(data, len(data))
@@ -193,7 +271,7 @@ class GTKClientWindowBase(ClientWindowBase, gtk.Window):
 
 
     def paint_spinner(self, context, area):
-        self.debug("paint_spinner(%s, %s)", context, area)
+        log("%s.paint_spinner(%s, %s)", self, context, area)
         #add grey semi-opaque layer on top:
         context.set_operator(cairo.OPERATOR_OVER)
         context.set_source_rgba(0.2, 0.2, 0.2, 0.8)
@@ -226,32 +304,40 @@ class GTKClientWindowBase(ClientWindowBase, gtk.Window):
 
 
     def do_map_event(self, event):
-        self.debug("Got map event: %s - OR=%s", event, self._override_redirect)
+        log("%s.do_map_event(%s) OR=%s", self, event, self._override_redirect)
         gtk.Window.do_map_event(self, event)
-        self._been_mapped = True
         xid = self._metadata.get("xid")
         if xid:
             self.set_xid(xid)
         if not self._override_redirect:
-            x, y, w, h = self.get_window_geometry()
-            if not self._been_mapped:
-                workspace = self.set_workspace()
-            else:
-                #window has been mapped, so these attributes can be read (if present):
-                self._client_properties["screen"] = self.get_screen().get_number()
-                workspace = self.get_window_workspace()
-                if workspace<0:
-                    workspace = self.get_current_workspace()
-            if workspace>=0:
-                self._client_properties["workspace"] = workspace
-            self.debug("map-window for wid=%s with client props=%s", self._id, self._client_properties)
-            self.send("map-window", self._id, x, y, w, h, self._client_properties)
-            self._pos = (x, y)
-            self._size = (w, h)
-            self.idle_add(self._focus_change, "initial")
+            self.process_map_event()
+        self._been_mapped = True
+
+    def process_map_event(self):
+        x, y, w, h = self.get_window_geometry()
+        props = self._client_properties
+        self._client_properties = {}
+        if not self._been_mapped:
+            workspace = self.set_workspace()
+        else:
+            #window has been mapped, so these attributes can be read (if present):
+            props["screen"] = self.get_screen().get_number()
+            workspace = self.get_window_workspace()
+            if workspace<0:
+                workspace = self.get_desktop_workspace()
+        if self._window_workspace!=workspace:
+            workspacelog("map event: changed workspace from %s to %s", self._window_workspace, workspace)
+            self._window_workspace = workspace
+            props["workspace"] = workspace
+        log("map-window for wid=%s with client props=%s", self._id, props)
+        self.send("map-window", self._id, x, y, w, h, props)
+        self._pos = (x, y)
+        self._size = (w, h)
+        self.idle_add(self._focus_change, "initial")
+
 
     def do_configure_event(self, event):
-        self.debug("Got configure event: %s", event)
+        log("%s.do_configure_event(%s)", self, event)
         gtk.Window.do_configure_event(self, event)
         if not self._override_redirect:
             self.process_configure_event()
@@ -263,48 +349,72 @@ class GTKClientWindowBase(ClientWindowBase, gtk.Window):
         ox, oy = self._pos
         dx, dy = x-ox, y-oy
         self._pos = (x, y)
-        if self._client.window_configure:
-            #if we support configure-window, send that first
-            if self._been_mapped:
-                #if the window has been mapped already, the workspace should be set:
-                self._client_properties["screen"] = self.get_screen().get_number()
-                workspace = self.get_window_workspace()
-                if workspace<0:
-                    workspace = self.get_current_workspace()
-                if workspace>=0:
-                    self._client_properties["workspace"] = workspace
-            self.debug("configure-window for wid=%s with client props=%s", self._id, self._client_properties)
-            self.send("configure-window", self._id, x, y, w, h, self._client_properties)
+        props = self._client_properties
+        self._client_properties = {}
+        if self._been_mapped:
+            #if the window has been mapped already, the workspace should be set:
+            props["screen"] = self.get_screen().get_number()
+            workspace = self.get_window_workspace()
+            if self._window_workspace!=workspace:
+                workspacelog("configure event: changed workspace from %s to %s", self._window_workspace, workspace)
+                self._window_workspace = workspace
+                props["workspace"] = workspace
+        packet = ["configure-window", self._id, x, y, w, h, props]
+        if self._resize_counter>0:
+            packet.append(self._resize_counter)
+        self.send(*packet)
         if dx!=0 or dy!=0:
-            #window has moved
-            if not self._client.window_configure:
-                #if we don't handle the move via configure:
-                self.send("move-window", self._id, x, y)
-            #move any OR window with their parent:
+            #window has moved, also move any child OR window:
             for window in self._override_redirect_windows:
                 x, y = window.get_position()
                 window.move(x+dx, y+dy)
         if (w, h) != self._size:
             self._size = (w, h)
             self.new_backing(w, h)
-            if not self._client.window_configure:
-                self.send("resize-window", self._id, w, h)
 
-    def move_resize(self, x, y, w, h):
-        assert self._override_redirect
+    def resize(self, w, h, resize_counter=0):
+        log("resize(%s, %s, %s)", w, h, resize_counter)
+        self._resize_counter = resize_counter
+        gtk.Window.resize(self, w, h)
+
+    def move_resize(self, x, y, w, h, resize_counter=0):
+        log("move_resize%s", (x, y, w, h, resize_counter))
         w = max(1, w)
         h = max(1, h)
-        self.window.move_resize(x, y, w, h)
-        if (w, h) != self._size:
+        self._resize_counter = resize_counter
+        if self.window.get_position()==(x, y):
+            #same location, just resize:
+            if self._size!=(w, h):
+                self.window.resize(w, h)
+        else:
+            mw, mh = gtk.gdk.get_default_root_window().get_size()
+            if not self.is_realized():
+                self.realize()
+            #adjust for window frame:
+            ox, oy = self.window.get_origin()
+            rx, ry = self.window.get_root_origin()
+            ax = x - (ox - rx)
+            ay = y - (oy - ry)
+            #validate against edge of screen (ensure window is shown):
+            if (ax + w)<0:
+                ax = -w + 1
+            elif ax >= mw:
+                ax = mw - 1
+            if (ay + h)<0:
+                ay = -y + 1
+            elif ay >= mh:
+                ay = mh -1
+            if self._size!=(w, h):
+                self.window.move_resize(ax, ay, w, h)
+            else:
+                #just move:
+                self.window.move(ax, ay)
+        if self._size!=(w, h):
             self._size = (w, h)
             self.new_backing(w, h)
 
     def destroy(self):
-        if self._refresh_timer:
-            self.source_remove(self._refresh_timer)
-        if self._backing:
-            self._backing.close()
-            self._backing = None
+        ClientWindowBase.destroy(self)
         gtk.Window.destroy(self)
         self._unfocus()
 
@@ -312,7 +422,7 @@ class GTKClientWindowBase(ClientWindowBase, gtk.Window):
     def do_unmap_event(self, event):
         self._unfocus()
         if not self._override_redirect:
-            self.send("unmap-window", self._id)
+            self.send("unmap-window", self._id, False)
 
     def do_delete_event(self, event):
         self.send("close-window", self._id)
@@ -341,6 +451,7 @@ class GTKClientWindowBase(ClientWindowBase, gtk.Window):
         key_event.group = event.group
         key_event.string = nn(event.string)
         key_event.pressed = pressed
+        keylog("parse_key_event(%s, %s)=%s", event, pressed, key_event)
         return key_event
 
     def do_key_press_event(self, event):
@@ -354,7 +465,7 @@ class GTKClientWindowBase(ClientWindowBase, gtk.Window):
 
     def _focus_change(self, *args):
         assert not self._override_redirect
-        htf = self.get_property("has-toplevel-focus")
-        self.debug("_focus_change(%s) wid=%s, has-toplevel-focus=%s, _been_mapped=%s", args, self._id, htf, self._been_mapped)
+        htf = self.has_toplevel_focus()
+        focuslog("%s focus_change(%s) has-toplevel-focus=%s, _been_mapped=%s", self, args, htf, self._been_mapped)
         if self._been_mapped:
             self._client.update_focus(self._id, htf)
