@@ -1,6 +1,6 @@
 # coding=utf8
 # This file is part of Xpra.
-# Copyright (C) 2011-2013 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2014 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -12,11 +12,9 @@ gobject = import_gobject()
 
 from xpra.gtk_common.gtk_util import set_tooltip_text, CheckMenuItem, ensure_item_selected, set_checkeditems, menuitem
 from xpra.client.gtk_base.about import about, close_about
-from xpra.client.gtk_base.session_info import SessionInfo
-from xpra.scripts.config import PREFERED_ENCODING_ORDER, ENCODINGS_HELP, ENCODINGS_TO_NAME
-from xpra.log import Logger, debug_if_env
-log = Logger()
-debug = debug_if_env(log, "XPRA_TRAY_DEBUG")
+from xpra.codecs.loader import PREFERED_ENCODING_ORDER, ENCODINGS_HELP, ENCODINGS_TO_NAME
+from xpra.log import Logger
+log = Logger("tray")
 
 
 #compression is fine with default value (3), no need to clutter the UI
@@ -90,7 +88,7 @@ def make_min_auto_menu(title, min_options, options, get_current_min_value, get_c
                 s = ts
                 break
         if s>=0 and s!=ss.get_current_value():
-            debug("setting %s to %s", title, s)
+            log("setting %s to %s", title, s)
             ss.set_value_cb(s)
             #deselect other items:
             for x in ss.menu_items.values():
@@ -115,7 +113,7 @@ def make_min_auto_menu(title, min_options, options, get_current_min_value, get_c
                 s = ts
                 break
         if s>=0 and s!=ss.get_current_min_value():
-            debug("setting min-%s to %s", title, s)
+            log("setting min-%s to %s", title, s)
             ss.set_min_value_cb(s)
             #deselect other min items:
             for x in ss.min_menu_items.values():
@@ -137,6 +135,10 @@ def make_min_auto_menu(title, min_options, options, get_current_min_value, get_c
 
 def make_encodingsmenu(get_current_encoding, set_encoding, encodings, server_encodings):
     encodings_submenu = gtk.Menu()
+    populate_encodingsmenu(encodings_submenu, get_current_encoding, set_encoding, encodings, server_encodings)
+    return encodings_submenu
+
+def populate_encodingsmenu(encodings_submenu, get_current_encoding, set_encoding, encodings, server_encodings):
     encodings_submenu.get_current_encoding = get_current_encoding
     encodings_submenu.set_encoding = set_encoding
     encodings_submenu.encodings = encodings
@@ -145,8 +147,6 @@ def make_encodingsmenu(get_current_encoding, set_encoding, encodings, server_enc
     encodings_submenu.encoding_to_index = {}
     NAME_TO_ENCODING = {}
     i = 0
-    cur = get_current_encoding()
-    cur = {"h264" : "x264", "vp8" : "vpx"}.get(cur, cur)
     for encoding in encodings:
         name = ENCODINGS_TO_NAME.get(encoding, encoding)
         descr = ENCODINGS_HELP.get(encoding)
@@ -157,13 +157,14 @@ def make_encodingsmenu(get_current_encoding, set_encoding, encodings, server_enc
                 descr += "\n(not available on this server)"
             set_tooltip_text(encoding_item, descr)
         def encoding_changed(oitem):
+            log("encoding_changed(%s)", oitem)
             item = ensure_item_selected(encodings_submenu, oitem)
             enc = NAME_TO_ENCODING.get(item.get_label())
-            debug("encoding_changed(%s) item=%s, enc=%s, current=%s", oitem, item, enc, encodings_submenu.get_current_encoding())
+            log("encoding_changed(%s) item=%s, enc=%s, current=%s", oitem, item, enc, encodings_submenu.get_current_encoding())
             if enc is not None and encodings_submenu.get_current_encoding()!=enc:
                 encodings_submenu.set_encoding(enc)
-        debug("make_encodingsmenu(..) encoding=%s, current=%s, active=%s", encoding, get_current_encoding(), encoding==get_current_encoding())
-        encoding_item.set_active(encoding==cur)
+        log("make_encodingsmenu(..) encoding=%s, current=%s, active=%s", encoding, get_current_encoding(), encoding==get_current_encoding())
+        encoding_item.set_active(encoding==get_current_encoding())
         encoding_item.set_sensitive(encoding in server_encodings)
         encoding_item.set_draw_as_radio(True)
         encoding_item.connect("toggled", encoding_changed)
@@ -172,7 +173,6 @@ def make_encodingsmenu(get_current_encoding, set_encoding, encodings, server_enc
         encodings_submenu.encoding_to_index[encoding] = i
         i += 1
     encodings_submenu.show_all()
-    return encodings_submenu
 
 
 class GTKTrayMenuBase(object):
@@ -190,14 +190,7 @@ class GTKTrayMenuBase(object):
         return self.menu
 
     def show_session_info(self, *args):
-        if self.session_info and not self.session_info.is_closed:
-            self.session_info.present()
-        else:
-            pixbuf = self.client.get_pixbuf("statistics.png")
-            if not pixbuf:
-                pixbuf = self.client.get_pixbuf("xpra.png")
-            self.session_info = SessionInfo(self.client, self.client.session_name, pixbuf, self.client._protocol._conn, self.client.get_pixbuf)
-            self.session_info.show_all()
+        self.client.show_session_info(*args)
 
     def get_image(self, *args):
         return self.client.get_image(*args)
@@ -250,7 +243,7 @@ class GTKTrayMenuBase(object):
         return menu
 
     def cleanup(self):
-        debug("cleanup() session_info=%s", self.session_info)
+        log("cleanup() session_info=%s", self.session_info)
         if self.session_info:
             self.session_info.destroy()
             self.session_info = None
@@ -308,7 +301,11 @@ class GTKTrayMenuBase(object):
         title = "Session Info"
         if self.client.session_name and self.client.session_name!="Xpra session":
             title = "Info: %s"  % self.client.session_name
-        return  self.handshake_menuitem(title, "statistics.png", None, self.show_session_info)
+        def show_session_info_cb(*args):
+            #we define a generic callback to remove the arguments
+            #(which contain the menu widget and are of no interest to the 'show_session_info' function)
+            self.show_session_info()
+        return  self.handshake_menuitem(title, "statistics.png", None, show_session_info_cb)
 
     def make_bellmenuitem(self):
         def bell_toggled(*args):
@@ -317,7 +314,7 @@ class GTKTrayMenuBase(object):
             self.client.bell_enabled = v
             if changed and self.client.toggle_cursors_bell_notify:
                 self.client.send_bell_enabled()
-            debug("bell_toggled(%s) bell_enabled=%s", args, self.client.bell_enabled)
+            log("bell_toggled(%s) bell_enabled=%s", args, self.client.bell_enabled)
         self.bell_menuitem = self.checkitem("Bell", bell_toggled)
         self.bell_menuitem.set_sensitive(False)
         def set_bell_menuitem(*args):
@@ -341,7 +338,7 @@ class GTKTrayMenuBase(object):
                 self.client.send_cursors_enabled()
             if not self.client.cursors_enabled:
                 self.client.reset_cursor()
-            debug("cursors_toggled(%s) cursors_enabled=%s", args, self.client.cursors_enabled)
+            log("cursors_toggled(%s) cursors_enabled=%s", args, self.client.cursors_enabled)
         self.cursors_menuitem = self.checkitem("Cursors", cursors_toggled)
         self.cursors_menuitem.set_sensitive(False)
         def set_cursors_menuitem(*args):
@@ -363,7 +360,7 @@ class GTKTrayMenuBase(object):
             self.client.notifications_enabled = v
             if changed and self.client.toggle_cursors_bell_notify:
                 self.client.send_notify_enabled()
-            debug("notifications_toggled(%s) notifications_enabled=%s", args, self.client.notifications_enabled)
+            log("notifications_toggled(%s) notifications_enabled=%s", args, self.client.notifications_enabled)
         self.notifications_menuitem = self.checkitem("Notifications", notifications_toggled)
         self.notifications_menuitem.set_sensitive(False)
         def set_notifications_menuitem(*args):
@@ -379,13 +376,13 @@ class GTKTrayMenuBase(object):
         return self.notifications_menuitem
 
     def make_clipboard_togglemenuitem(self):
-        def clipboard_toggled(*args):
+        def menu_clipboard_toggled(*args):
             new_state = self.clipboard_menuitem.get_active()
-            debug("clipboard_toggled(%s) clipboard_enabled=%s, new_state=%s", args, self.client.clipboard_enabled, new_state)
+            log("clipboard_toggled(%s) clipboard_enabled=%s, new_state=%s", args, self.client.clipboard_enabled, new_state)
             if self.client.clipboard_enabled!=new_state:
                 self.client.clipboard_enabled = new_state
                 self.client.emit("clipboard-toggled")
-        self.clipboard_menuitem = self.checkitem("Clipboard", clipboard_toggled)
+        self.clipboard_menuitem = self.checkitem("Clipboard", menu_clipboard_toggled)
         self.clipboard_menuitem.set_sensitive(False)
         def set_clipboard_menuitem(*args):
             self.clipboard_menuitem.set_active(self.client.clipboard_enabled)
@@ -397,6 +394,11 @@ class GTKTrayMenuBase(object):
             else:
                 set_tooltip_text(self.clipboard_menuitem, "Clipboard synchronization cannot be enabled: disabled by server")
         self.client.connect("handshake-complete", set_clipboard_menuitem)
+        def clipboard_toggled(*args):
+            #keep menu in sync with actual "clipboard_enabled" flag:
+            if self.client.clipboard_enabled != self.clipboard_menuitem.get_active():
+                self.clipboard_menuitem.set_active(self.client.clipboard_enabled)
+        self.client.connect("clipboard-toggled", clipboard_toggled)
         return self.clipboard_menuitem
 
     def make_translatedclipboard_optionsmenuitem(self):
@@ -408,7 +410,7 @@ class GTKTrayMenuBase(object):
             self.popup_menu_workaround(clipboard_submenu)
             c = self.client
             can_clipboard = c.server_supports_clipboard and c.client_supports_clipboard and c.server_supports_clipboard
-            debug("set_clipboard_menu(%s) can_clipboard=%s, server=%s, client=%s", args, can_clipboard, c.server_supports_clipboard, c.client_supports_clipboard)
+            log("set_clipboard_menu(%s) can_clipboard=%s, server=%s, client=%s", args, can_clipboard, c.server_supports_clipboard, c.client_supports_clipboard)
             clipboard_menu.set_sensitive(can_clipboard)
             LABEL_TO_NAME = {"Disabled"  : None,
                             "Clipboard" : "CLIPBOARD",
@@ -423,7 +425,7 @@ class GTKTrayMenuBase(object):
                     label = item.get_label()
                     remote_clipboard = LABEL_TO_NAME.get(label)
                     old_state = self.client.clipboard_enabled
-                    debug("remote_clipboard_changed(%s) remote_clipboard=%s, old_state=%s", item, remote_clipboard, old_state)
+                    log("remote_clipboard_changed(%s) remote_clipboard=%s, old_state=%s", item, remote_clipboard, old_state)
                     send_tokens = False
                     if remote_clipboard is not None:
                         #clipboard is not disabled
@@ -436,7 +438,7 @@ class GTKTrayMenuBase(object):
                         self.client.clipboard_helper = None
                         send_tokens = False
                         new_state = False
-                    debug("remote_clipboard_changed(%s) label=%s, remote_clipboard=%s, old_state=%s, new_state=%s",
+                    log("remote_clipboard_changed(%s) label=%s, remote_clipboard=%s, old_state=%s, new_state=%s",
                              item, label, remote_clipboard, old_state, new_state)
                     if new_state!=old_state:
                         self.client.clipboard_enabled = new_state
@@ -478,7 +480,7 @@ class GTKTrayMenuBase(object):
                 set_tooltip_text(self.keyboard_sync_menuitem, "Enable keyboard state synchronization")
         def keyboard_sync_toggled(*args):
             self.client.keyboard_sync = self.keyboard_sync_menuitem.get_active()
-            debug("keyboard_sync_toggled(%s) keyboard_sync=%s", args, self.client.keyboard_sync)
+            log("keyboard_sync_toggled(%s) keyboard_sync=%s", args, self.client.keyboard_sync)
             set_keyboard_sync_tooltip()
             self.client.emit("keyboard-sync-toggled")
         self.keyboard_sync_menuitem = self.checkitem("Keyboard Synchronization", keyboard_sync_toggled)
@@ -493,7 +495,7 @@ class GTKTrayMenuBase(object):
     def make_openglmenuitem(self):
         gl = self.checkitem("OpenGL")
         def gl_set(*args):
-            debug("gl_set(%s) opengl_enabled=%s, window_unmap=%s", args, self.client.opengl_enabled, self.client.window_unmap)
+            log("gl_set(%s) opengl_enabled=%s, window_unmap=%s", args, self.client.opengl_enabled, self.client.window_unmap)
             gl.set_active(self.client.opengl_enabled)
             gl.set_sensitive(self.client.window_unmap)
             if not self.client.window_unmap:
@@ -629,17 +631,17 @@ class GTKTrayMenuBase(object):
                 set_tooltip_text(self.speed, "Quality is always 100% with mmap")
             elif not self.client.change_speed:
                 set_tooltip_text(self.speed, "Server does not support changing speed")
-            elif self.client.encoding!="x264":
+            elif self.client.encoding!="h264":
                 set_tooltip_text(self.speed, "Not supported with %s encoding" % self.client.encoding)
             else:
                 set_tooltip_text(self.speed, "Encoding latency vs size")
 
 
     def spk_on(self, *args):
-        debug("spk_on(%s)", args)
+        log("spk_on(%s)", args)
         self.client.start_receiving_sound()
     def spk_off(self, *args):
-        debug("spk_off(%s)", args)
+        log("spk_off(%s)", args)
         self.client.stop_receiving_sound()
     def make_speakermenuitem(self):
         speaker = self.menuitem("Speaker", "speaker.png", "Forward sound output from the server")
@@ -657,10 +659,10 @@ class GTKTrayMenuBase(object):
         return speaker
 
     def mic_on(self, *args):
-        debug("mic_on(%s)", args)
+        log("mic_on(%s)", args)
         self.client.start_sending_sound()
     def mic_off(self, *args):
-        debug("mic_off(%s)", args)
+        log("mic_off(%s)", args)
         self.client.stop_sending_sound()
     def make_microphonemenuitem(self):
         microphone = self.menuitem("Microphone", "microphone.png", "Forward sound input to the server", None)
@@ -701,7 +703,7 @@ class GTKTrayMenuBase(object):
         def client_signalled_change(obj):
             menu.ignore_events = True
             is_on = is_on_cb()
-            debug("sound: client_signalled_change(%s) is_on=%s", obj, is_on)
+            log("sound: client_signalled_change(%s) is_on=%s", obj, is_on)
             if is_on:
                 if not on.get_active():
                     on.set_active(True)
@@ -731,7 +733,7 @@ class GTKTrayMenuBase(object):
                 layout = item.keyboard_layout
                 variant = item.keyboard_variant
                 if layout!=self.client.xkbmap_layout or variant!=self.client.xkbmap_variant:
-                    debug("keyboard layout selected: %s / %s", layout, variant)
+                    log("keyboard layout selected: %s / %s", layout, variant)
                     self.client.xkbmap_layout = layout
                     self.client.xkbmap_variant = variant
                     self.client.send_layout()
@@ -798,7 +800,7 @@ class GTKTrayMenuBase(object):
             item = ensure_item_selected(self.compression_submenu, item)
             c = int(item.get_label().replace("None", "0"))
             if c!=self.client.compression_level:
-                debug("setting compression level to %s", c)
+                log("setting compression level to %s", c)
                 self.client.set_deflate_level(c)
         for i in range(0, 10):
             c = CheckMenuItem(str(compression_options.get(i, i)))
@@ -815,7 +817,7 @@ class GTKTrayMenuBase(object):
 
     def make_refreshmenuitem(self):
         def force_refresh(*args):
-            debug("force refresh")
+            log("force refresh")
             self.client.send_refresh_all()
         return self.handshake_menuitem("Refresh", "retry.png", None, force_refresh)
 
@@ -847,11 +849,11 @@ class GTKTrayMenuBase(object):
             This code must be added to all the sub-menus of the popup menu too!
         """
         def enter_menu(*args):
-            debug("mouse_in_tray_menu=%s", self.mouse_in_tray_menu)
+            log("mouse_in_tray_menu=%s", self.mouse_in_tray_menu)
             self.mouse_in_tray_menu_counter += 1
             self.mouse_in_tray_menu = True
         def leave_menu(*args):
-            debug("mouse_in_tray_menu=%s", self.mouse_in_tray_menu)
+            log("mouse_in_tray_menu=%s", self.mouse_in_tray_menu)
             self.mouse_in_tray_menu_counter += 1
             self.mouse_in_tray_menu = False
             def check_menu_left(expected_counter):
@@ -863,6 +865,6 @@ class GTKTrayMenuBase(object):
             gobject.timeout_add(500, check_menu_left, self.mouse_in_tray_menu_counter)
         self.mouse_in_tray_menu_counter = 0
         self.mouse_in_tray_menu = False
-        debug("popup_menu_workaround: adding events callbacks")
+        log("popup_menu_workaround: adding events callbacks")
         menu.connect("enter-notify-event", enter_menu)
         menu.connect("leave-notify-event", leave_menu)
