@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2012, 2013 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2012, 2014 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -10,14 +10,15 @@ gobject = import_gobject()
 
 from xpra.client.window_backing_base import WindowBackingBase
 from xpra.log import Logger
-log = Logger()
+log = Logger("window", "paint")
 
 
 """
-Generic superclass for Backing code,
+Generic GTK superclass for Backing code (for both GTK2 and GTK3),
 see CairoBacking and PixmapBacking for actual implementations
 """
 class GTKWindowBacking(WindowBackingBase):
+
     def __init__(self, wid):
         WindowBackingBase.__init__(self, wid, gobject.idle_add)
 
