@@ -1,6 +1,6 @@
 # coding=utf8
 # This file is part of Xpra.
-# Copyright (C) 2011-2013 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2014 Antoine Martin <antoine@devloop.org.uk>
 # Copyright (C) 2010 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -13,14 +13,15 @@ import sys
 import time
 import datetime
 
-from xpra.os_util import platform_name
+from xpra.os_util import os_info
 from xpra.gtk_common.graph import make_graph_pixmap
 from xpra.deque import maxdeque
 from xpra.simple_stats import values_to_scaled_values, values_to_diff_scaled_values, to_std_unit, std_unit_dec
-from xpra.scripts.config import HAS_SOUND, python_platform
+from xpra.scripts.config import python_platform
 from xpra.log import Logger
 from xpra.gtk_common.gtk_util import add_close_accel, label, title_box, set_tooltip_text, TableBuilder, imagebutton
-log = Logger()
+from xpra.net.protocol import get_network_caps
+log = Logger("info")
 
 N_SAMPLES = 20      #how many sample points to show on the graphs
 SHOW_PIXEL_STATS = True
@@ -73,7 +74,7 @@ class SessionInfo(gtk.Window):
         self.last_populate_statistics = 0
         self.is_closed = False
         self.get_pixbuf = get_pixbuf
-        if self.session_name=="Xpra":
+        if not self.session_name or self.session_name=="Xpra":
             title = "Session Info"
         else:
             title = "%s: Session Info" % self.session_name
@@ -96,21 +97,20 @@ class SessionInfo(gtk.Window):
         self.tab_box.pack_start(self.tab_button_box, expand=False, fill=True, padding=0)
 
         #Package Table:
-        tb = self.table_tab("package.png", "Software", self.populate_package)
+        tb, _ = self.table_tab("package.png", "Software", self.populate_package)
         #title row:
         tb.attach(title_box(""), 0, xoptions=gtk.EXPAND|gtk.FILL, xpadding=0)
         tb.attach(title_box("Client"), 1, xoptions=gtk.EXPAND|gtk.FILL, xpadding=0)
         tb.attach(title_box("Server"), 2, xoptions=gtk.EXPAND|gtk.FILL, xpadding=0)
         tb.inc()
 
-        def make_os_str(sys_platform, platform_release, platform_platform, platform_linux_distribution):
-            s = platform_name(sys_platform, platform_release)
-            if platform_linux_distribution and len(platform_linux_distribution)==3 and len(platform_linux_distribution[0])>0:
-                s += "\n%s" % (" ".join(platform_linux_distribution))
-            elif platform_platform:
-                s += "\n%s" % platform_platform
-            return s
-        LOCAL_PLATFORM_NAME = make_os_str(sys.platform, python_platform.release(), python_platform.platform(), python_platform.linux_distribution())
+        def make_os_str(*args):
+            s = os_info(*args)
+            return "\n".join(s)
+        distro = ""
+        if hasattr(python_platform, "linux_distribution"):
+            distro = python_platform.linux_distribution()
+        LOCAL_PLATFORM_NAME = make_os_str(sys.platform, python_platform.release(), python_platform.platform(), distro)
         SERVER_PLATFORM_NAME = make_os_str(self.client._remote_platform, self.client._remote_platform_release, self.client._remote_platform_platform, self.client._remote_platform_linux_distribution)
         tb.new_row("Operating System", label(LOCAL_PLATFORM_NAME), label(SERVER_PLATFORM_NAME))
         scaps = self.client.server_capabilities
@@ -119,7 +119,7 @@ class SessionInfo(gtk.Window):
         cl_rev, cl_ch, cl_date = "unknown", "", ""
         try:
             from xpra.build_info import BUILD_DATE as cl_date
-            from xpra.src_info import REVISION as cl_rev, LOCAL_MODIFICATIONS as cl_ch
+            from xpra.src_info import REVISION as cl_rev, LOCAL_MODIFICATIONS as cl_ch      #@UnresolvedImport
         except:
             pass
         def make_version_str(version):
@@ -131,6 +131,10 @@ class SessionInfo(gtk.Window):
                 v = scaps.get(x)
                 if v is not None:
                     return v
+                if self.client.server_last_info:
+                    v = self.client.server_last_info.get(x)
+                if v is not None:
+                    return v
             return None
         def server_version_info(*prop_names):
             return make_version_str(server_info(*prop_names))
@@ -149,13 +153,13 @@ class SessionInfo(gtk.Window):
         else:
             tb.new_row("PyGTK", label(client_version_info("pygtk_version")), label(server_version_info("pygtk.version", "pygtk_version")))
             tb.new_row("GTK", label(client_version_info("gtk_version")), label(server_version_info("gtk.version", "gtk_version")))
-        tb.new_row("Python", label(python_platform.python_version()), label(server_version_info("server.python.version", "python_version")))
+        tb.new_row("Python", label(python_platform.python_version()), label(server_version_info("server.python.version", "python_version", "python.version")))
+
         cl_gst_v, cl_pygst_v = "", ""
-        if HAS_SOUND:
-            try:
-                from xpra.sound.gstreamer_util import gst_version as cl_gst_v, pygst_version as cl_pygst_v
-            except:
-                pass
+        try:
+            from xpra.sound.gstreamer_util import gst_version as cl_gst_v, pygst_version as cl_pygst_v
+        except Exception, e:
+            log("cannot load gstreamer: %s", e)
         tb.new_row("GStreamer", label(make_version_str(cl_gst_v)), label(server_version_info("sound.gst.version", "gst_version")))
         tb.new_row("pygst", label(make_version_str(cl_pygst_v)), label(server_version_info("sound.pygst.version", "pygst_version")))
         tb.new_row("OpenGL", label(make_version_str(self.client.opengl_props.get("opengl", "n/a"))), label("n/a"))
@@ -163,7 +167,14 @@ class SessionInfo(gtk.Window):
         tb.new_row("PyOpenGL", label(make_version_str(self.client.opengl_props.get("pyopengl", "n/a"))), label("n/a"))
 
         # Features Table:
-        tb = self.table_tab("features.png", "Features", self.populate_features)
+        vbox = self.vbox_tab("features.png", "Features", self.populate_features)
+        #add top table:
+        tb = TableBuilder(rows=1, columns=2)
+        table = tb.get_table()
+        al = gtk.Alignment(xalign=0.5, yalign=0.5, xscale=0.0, yscale=1.0)
+        al.add(table)
+        vbox.pack_start(al, expand=True, fill=False, padding=10)
+        #top table contents:
         randr_box = gtk.HBox(False, 20)
         self.server_randr_label = label()
         self.server_randr_icon = gtk.Image()
@@ -179,10 +190,6 @@ class SessionInfo(gtk.Window):
         tb.new_row("Client OpenGL", opengl_box)
         self.opengl_buffering = label()
         tb.new_row("OpenGL Buffering", self.opengl_buffering)
-        self.server_encodings_label = label()
-        tb.new_row("Server Encodings", self.server_encodings_label)
-        self.client_encodings_label = label()
-        tb.new_row("Client Encodings", self.client_encodings_label)
         self.server_mmap_icon = gtk.Image()
         tb.new_row("Memory Mapped Transfers", self.server_mmap_icon)
         self.server_clipboard_icon = gtk.Image()
@@ -199,23 +206,44 @@ class SessionInfo(gtk.Window):
         self.speaker_codec_label = label()
         speaker_box.add(self.speaker_codec_label)
         tb.new_row("Speaker Forwarding", speaker_box)
-        self.server_speaker_codecs_label = label()
-        tb.new_row("Server Codecs", self.server_speaker_codecs_label)
-        self.client_speaker_codecs_label = label()
-        tb.new_row("Client Codecs", self.client_speaker_codecs_label)
         microphone_box = gtk.HBox(False, 20)
         self.server_microphone_icon = gtk.Image()
         microphone_box.add(self.server_microphone_icon)
         self.microphone_codec_label = label()
         microphone_box.add(self.microphone_codec_label)
         tb.new_row("Microphone Forwarding", microphone_box)
-        self.server_microphone_codecs_label = label()
-        tb.new_row("Server Codecs", self.server_microphone_codecs_label)
+        #add bottom table:
+        tb = TableBuilder(rows=1, columns=3)
+        table = tb.get_table()
+        vbox.pack_start(table, expand=True, fill=True, padding=20)
+        #bottom table headings:
+        tb.attach(title_box(""), 0, xoptions=gtk.EXPAND|gtk.FILL, xpadding=0)
+        tb.attach(title_box("Client"), 1, xoptions=gtk.EXPAND|gtk.FILL, xpadding=0)
+        tb.attach(title_box("Server"), 2, xoptions=gtk.EXPAND|gtk.FILL, xpadding=0)
+        tb.inc()
+        #bottom table contents:
+        self.client_encodings_label = label()
+        self.client_encodings_label.set_line_wrap(True)
+        self.client_encodings_label.set_size_request(250, -1)
+        self.server_encodings_label = label()
+        self.server_encodings_label.set_line_wrap(True)
+        self.server_encodings_label.set_size_request(250, -1)
+        tb.new_row("Encodings", self.client_encodings_label, self.server_encodings_label)
+        self.client_speaker_codecs_label = label()
+        self.server_speaker_codecs_label = label()
+        tb.new_row("Speaker Codecs", self.client_speaker_codecs_label, self.server_speaker_codecs_label)
         self.client_microphone_codecs_label = label()
-        tb.new_row("Client Codecs", self.client_microphone_codecs_label)
+        self.server_microphone_codecs_label = label()
+        tb.new_row("Microphone Codecs", self.client_microphone_codecs_label, self.server_microphone_codecs_label)
+        self.client_packet_encoders_label = label()
+        self.server_packet_encoders_label = label()
+        tb.new_row("Packet Encoders", self.client_packet_encoders_label, self.server_packet_encoders_label)
+        self.client_packet_compressors_label = label()
+        self.server_packet_compressors_label = label()
+        tb.new_row("Packet Compressors", self.client_packet_compressors_label, self.server_packet_compressors_label)
 
         # Connection Table:
-        tb = self.table_tab("connect.png", "Connection", self.populate_connection)
+        tb, _ = self.table_tab("connect.png", "Connection", self.populate_connection)
         tb.new_row("Server Endpoint", label(self.connection.target))
         if self.client.server_display:
             tb.new_row("Server Display", label(self.client.server_display))
@@ -238,7 +266,7 @@ class SessionInfo(gtk.Window):
         self.output_bytes_label = label()
         tb.new_row("Bytes Sent", self.output_bytes_label)
         self.compression_label = label()
-        tb.new_row("Compression Level", self.compression_label)
+        tb.new_row("Compression + Encoding", self.compression_label)
         self.connection_type_label = label()
         tb.new_row("Connection Type", self.connection_type_label)
         self.input_encryption_label = label()
@@ -253,7 +281,7 @@ class SessionInfo(gtk.Window):
         tb.new_row("Microphone", self.microphone_label)
 
         # Details:
-        tb = self.table_tab("browse.png", "Statistics", self.populate_statistics)
+        tb, stats_box = self.table_tab("browse.png", "Statistics", self.populate_statistics)
         tb.widget_xalign = 1.0
         tb.attach(title_box(""), 0, xoptions=gtk.EXPAND|gtk.FILL, xpadding=0)
         tb.attach(title_box("Latest"), 1, xoptions=gtk.EXPAND|gtk.FILL, xpadding=0)
@@ -266,46 +294,69 @@ class SessionInfo(gtk.Window):
         def maths_labels():
             return label(), label(), label(), label(), label()
         self.server_latency_labels = maths_labels()
-        tb.add_row(label("Server Latency (ms)"), *self.server_latency_labels)
+        tb.add_row(label("Server Latency (ms)", "The time it takes for the server to respond to pings"),
+                   *self.server_latency_labels)
         self.client_latency_labels = maths_labels()
-        tb.add_row(label("Client Latency (ms)"), *self.client_latency_labels)
+        tb.add_row(label("Client Latency (ms)", "The time it takes for the client to respond to pings, as measured by the server"),
+                   *self.client_latency_labels)
         if self.client.windows_enabled:
             if self.client.server_info_request:
                 self.batch_labels = maths_labels()
-                tb.add_row(label("Batch Delay (ms)"), *self.batch_labels)
+                tb.add_row(label("Batch Delay (ms)", "How long the server waits for new screen updates to accumulate before processing them"),
+                           *self.batch_labels)
                 self.damage_labels = maths_labels()
-                tb.add_row(label("Damage Latency (ms)"), *self.damage_labels)
+                tb.add_row(label("Damage Latency (ms)", "The time it takes to compress a frame and pass it to the OS network layer"),
+                           *self.damage_labels)
                 self.quality_labels = maths_labels()
                 tb.add_row(label("Encoding Quality (pct)"), *self.quality_labels)
                 self.speed_labels = maths_labels()
                 tb.add_row(label("Encoding Speed (pct)"), *self.speed_labels)
 
             self.decoding_labels = maths_labels()
-            tb.add_row(label("Decoding Latency (ms)"), *self.decoding_labels)
+            tb.add_row(label("Decoding Latency (ms)", "How long it takes the client to decode a screen update"), *self.decoding_labels)
             self.regions_per_second_labels = maths_labels()
-            tb.add_row(label("Regions/s"), *self.regions_per_second_labels)
+            tb.add_row(label("Regions/s", "The number of screen updates per second (includes both partial and full screen updates)"), *self.regions_per_second_labels)
             self.regions_sizes_labels = maths_labels()
-            tb.add_row(label("Pixels/region"), *self.regions_sizes_labels)
+            tb.add_row(label("Pixels/region", "The number of pixels per screen update"), *self.regions_sizes_labels)
             self.pixels_per_second_labels = maths_labels()
-            tb.add_row(label("Pixels/s"), *self.pixels_per_second_labels)
+            tb.add_row(label("Pixels/s", "The number of pixels processed per second"), *self.pixels_per_second_labels)
 
+            #Window count stats:
+            wtb = TableBuilder()
+            stats_box.add(wtb.get_table())
+            #title row:
+            wtb.attach(title_box(""), 0, xoptions=gtk.EXPAND|gtk.FILL, xpadding=0)
+            wtb.attach(title_box("Regular"), 1, xoptions=gtk.EXPAND|gtk.FILL, xpadding=0)
+            wtb.attach(title_box("Transient"), 2, xoptions=gtk.EXPAND|gtk.FILL, xpadding=0)
+            wtb.attach(title_box("Trays"), 3, xoptions=gtk.EXPAND|gtk.FILL, xpadding=0)
+            if self.client.client_supports_opengl:
+                wtb.attach(title_box("OpenGL"), 4, xoptions=gtk.EXPAND|gtk.FILL, xpadding=0)
+            wtb.inc()
+
+            wtb.attach(label("Windows:"), 0, xoptions=gtk.EXPAND|gtk.FILL, xpadding=0)
             self.windows_managed_label = label()
-            tb.new_row("Regular Windows", self.windows_managed_label),
+            wtb.attach(self.windows_managed_label, 1)
             self.transient_managed_label = label()
-            tb.new_row("Transient Windows", self.transient_managed_label),
+            wtb.attach(self.transient_managed_label, 2)
             self.trays_managed_label = label()
-            tb.new_row("Trays Managed", self.trays_managed_label),
+            wtb.attach(self.trays_managed_label, 3)
             if self.client.client_supports_opengl:
                 self.opengl_label = label()
-                tb.new_row("OpenGL Windows", self.opengl_label),
+                wtb.attach(self.opengl_label, 4)
+
+            #add encoder info:
+            etb = TableBuilder()
+            stats_box.add(etb.get_table())
+            self.encoder_info_box = gtk.HBox(spacing=4)
+            etb.new_row("Window Encoders", self.encoder_info_box)
 
         self.graph_box = gtk.VBox(False, 10)
         self.add_tab("statistics.png", "Graphs", self.populate_graphs, self.graph_box)
-        bandwidth_label = "Number of bytes measured by the networks sockets"
+        bandwidth_label = "Bandwidth used"
         if SHOW_PIXEL_STATS:
             bandwidth_label += ",\nand number of pixels rendered"
         self.bandwidth_graph = self.add_graph_button(bandwidth_label, self.save_graphs)
-        self.latency_graph = self.add_graph_button("The time it takes to send an echo packet and get the reply", self.save_graphs)
+        self.latency_graph = self.add_graph_button(None, self.save_graphs)
         self.pixel_in_data = maxdeque(N_SAMPLES+4)
         self.net_in_bytecount = maxdeque(N_SAMPLES+4)
         self.net_out_bytecount = maxdeque(N_SAMPLES+4)
@@ -317,8 +368,9 @@ class SessionInfo(gtk.Window):
         def window_deleted(*args):
             self.is_closed = True
         self.connect('delete_event', window_deleted)
-        self.show_tab(self.tabs[0][1])
+        self.show_tab(self.tabs[0][2])
         self.set_size_request(-1, 480)
+        self.init_counters()
         self.populate()
         self.populate_all()
         gobject.timeout_add(1000, self.populate)
@@ -330,12 +382,17 @@ class SessionInfo(gtk.Window):
     def table_tab(self, icon_filename, title, populate_cb):
         tb = TableBuilder()
         table = tb.get_table()
-        box = gtk.VBox(False, 0)
+        vbox = self.vbox_tab(icon_filename, title, populate_cb)
         al = gtk.Alignment(xalign=0.5, yalign=0.5, xscale=0.0, yscale=1.0)
         al.add(table)
-        box.pack_start(al, expand=True, fill=True, padding=20)
-        self.add_tab(icon_filename, title, populate_cb, contents=box)
-        return tb
+        vbox.pack_start(al, expand=True, fill=True, padding=20)
+        return tb, vbox
+
+    def vbox_tab(self, icon_filename, title, populate_cb):
+        vbox = gtk.VBox(False, 0)
+        self.add_tab(icon_filename, title, populate_cb, contents=vbox)
+        return vbox
+
 
     def add_tab(self, icon_filename, title, populate_cb, contents):
         icon = self.get_pixbuf(icon_filename)
@@ -345,11 +402,11 @@ class SessionInfo(gtk.Window):
         button.connect("clicked", show_tab)
         button.set_relief(gtk.RELIEF_NONE)
         self.tab_button_box.add(button)
-        self.tabs.append((button, contents, populate_cb))
+        self.tabs.append((title, button, contents, populate_cb))
 
     def show_tab(self, table):
         button = None
-        for b, t, p_cb in self.tabs:
+        for _, b, t, p_cb in self.tabs:
             if t==table:
                 button = b
                 b.set_relief(gtk.RELIEF_NORMAL)
@@ -364,8 +421,25 @@ class SessionInfo(gtk.Window):
         self.tab_box.pack_start(table, expand=True, fill=True, padding=0)
         table.show_all()
 
+
+    def set_args(self, *args):
+        #this is a generic way for keyboard shortcuts or remote commands
+        #to pass parameters to us
+        log("set_args%s", args)
+        if len(args)==0:
+            return
+        #at the moment, we only handle the tab name as argument:
+        tab_name = args[0]
+        if tab_name.lower()!="help":
+            for title, _, table, _ in self.tabs:
+                if title.lower()==tab_name.lower():
+                    self.show_tab(table)
+                    return
+            log.warn("could not find session info tab named: %s", title)
+        log.warn("The options for tab names are: %s)", [x[0] for x in self.tabs])
+
     def populate_all(self):
-        for _, _, p_cb in self.tabs:
+        for _, _, _, p_cb in self.tabs:
             if p_cb:
                 p_cb()
 
@@ -383,7 +457,8 @@ class SessionInfo(gtk.Window):
         graph.set_size_request(0, 0)
         button.connect("button_press_event", click_cb)
         button.add(graph)
-        set_tooltip_text(graph, tooltip)
+        if tooltip:
+            set_tooltip_text(graph, tooltip)
         self.graph_box.add(button)
         return graph
 
@@ -435,8 +510,43 @@ class SessionInfo(gtk.Window):
             return [recs.get(x) for x in sorted(recs.keys())]
         self.server_latency = get_ping_latency_records(self.client.server_ping_latency)
         self.client_latency = get_ping_latency_records(self.client.client_ping_latency)
+        if self.client.server_last_info:
+            #populate running averages for graphs:
+            def getavg(name):
+                return self.client.server_last_info.get("%s.avg" % name)
+            def addavg(l, name):
+                v = getavg(name)
+                if v:
+                    l.append(v)
+            addavg(self.avg_batch_delay, "batch.delay")
+            addavg(self.avg_damage_out_latency, "damage.out_latency")
+            if len(self.client.server_ping_latency)>0 and len(self.client.client_ping_latency)>0:
+                spl = [1000.0*x for _,x in list(self.client.server_ping_latency)]
+                cpl = [1000.0*x for _,x in list(self.client.client_ping_latency)]
+                self.avg_ping_latency.append(sum(spl+cpl)/len(spl+cpl))
+            if len(self.client.pixel_counter)>0:
+                tsize = 0
+                ttime = 0
+                for start_time, end_time, size in self.client.pixel_counter:
+                    ttime += 1000.0 * (end_time-start_time) * size
+                    tsize += size
+                self.avg_decoding_latency.append(int(ttime/tsize))
+        #totals: ping latency is halved since we only care about sending, not sending+receiving
+        els  = [(self.avg_batch_delay, 1), (self.avg_damage_out_latency, 1),
+                (self.avg_ping_latency, 2), (self.avg_decoding_latency, 1)]
+        if len([x for x, _ in els if len(x)>0])==len(els):
+            totals = [x[-1]/r for x, r in els]
+            log("frame totals=%s", totals)
+            self.avg_total.append(sum(totals))
         return not self.is_closed
 
+    def init_counters(self):
+        self.avg_batch_delay = maxdeque(N_SAMPLES+4)
+        self.avg_damage_out_latency = maxdeque(N_SAMPLES+4)
+        self.avg_ping_latency = maxdeque(N_SAMPLES+4)
+        self.avg_decoding_latency = maxdeque(N_SAMPLES+4)
+        self.avg_total = maxdeque(N_SAMPLES+4)
+
     def populate_tab(self, *args):
         if self.is_closed:
             return False
@@ -475,9 +585,6 @@ class SessionInfo(gtk.Window):
         self.opengl_buffering.set_text(buffering)
 
         scaps = self.client.server_capabilities
-        se = scaps.get("encodings.core", scaps.get("encodings", []))
-        self.server_encodings_label.set_text(", ".join(sorted(se)))
-        self.client_encodings_label.set_text(", ".join(sorted(self.client.get_core_encodings())))
         self.bool_icon(self.server_mmap_icon, self.client.mmap_enabled)
         self.bool_icon(self.server_clipboard_icon, scaps.get("clipboard", False))
         self.bool_icon(self.server_notifications_icon, scaps.get("notifications", False))
@@ -503,18 +610,43 @@ class SessionInfo(gtk.Window):
             can = scaps.get("sound.send", False) and self.client.speaker_allowed
             self.bool_icon(self.server_speaker_icon, can)
             self.speaker_codec_label.set_text(pipeline_info(can, self.client.sound_sink))
-            self.server_speaker_codecs_label.set_text(codec_info(scaps.get("sound.send", False), scaps.get("sound.encoders", [])))
-            self.client_speaker_codecs_label.set_text(codec_info(self.client.speaker_allowed, self.client.speaker_codecs))
         populate_speaker_info()
         self.client.connect("speaker-changed", populate_speaker_info)
         def populate_microphone_info(*args):
             can = scaps.get("sound.receive", False) and self.client.microphone_allowed
             self.bool_icon(self.server_microphone_icon, can)
             self.microphone_codec_label.set_text(pipeline_info(can, self.client.sound_source))
-            self.server_microphone_codecs_label.set_text(codec_info(scaps.get("sound.receive", False), scaps.get("sound.decoders", [])))
-            self.client_microphone_codecs_label.set_text(codec_info(self.client.microphone_allowed, self.client.microphone_codecs))
         populate_microphone_info()
         self.client.connect("microphone-changed", populate_microphone_info)
+
+        #sound/video codec table:
+        self.server_speaker_codecs_label.set_text(codec_info(scaps.get("sound.send", False), scaps.get("sound.encoders", [])))
+        self.client_speaker_codecs_label.set_text(codec_info(self.client.speaker_allowed, self.client.speaker_codecs))
+        self.server_microphone_codecs_label.set_text(codec_info(scaps.get("sound.receive", False), scaps.get("sound.decoders", [])))
+        self.client_microphone_codecs_label.set_text(codec_info(self.client.microphone_allowed, self.client.microphone_codecs))
+        se = scaps.get("encodings.core", scaps.get("encodings", scaps.get("encodings.core", [])))
+        self.server_encodings_label.set_text(", ".join(sorted(se)))
+        self.client_encodings_label.set_text(", ".join(sorted(self.client.get_core_encodings())))
+
+        def get_encoder_list(caps):
+            l = []
+            if caps.get("bencode", True):
+                l.append("bencode")
+            if caps.get("rencode", False):
+                l.append("rencode")
+            return l
+        self.client_packet_encoders_label.set_text(", ".join(get_encoder_list(get_network_caps())))
+        self.server_packet_encoders_label.set_text(", ".join(get_encoder_list(self.client.server_capabilities)))
+
+        def get_compressor_list(caps):
+            l = []
+            if caps.get("zlib", False):
+                l.append("zlib")
+            if caps.get("lz4", False):
+                l.append("lz4")
+            return l
+        self.client_packet_compressors_label.set_text(", ".join(get_compressor_list(get_network_caps())))
+        self.server_packet_compressors_label.set_text(", ".join(get_compressor_list(self.client.server_capabilities)))
         return False
 
     def populate_connection(self):
@@ -530,6 +662,9 @@ class SessionInfo(gtk.Window):
         settimedeltastr(self.session_connected_label, self.client.start_time)
 
         p = self.client._protocol
+        if p is None:
+            #no longer connected!
+            return False
         c = p._conn
         self.input_packets_label.set_text(std_unit_dec(p.input_packetcount))
         self.input_bytes_label.set_text(std_unit_dec(c.input_bytecount))
@@ -555,14 +690,65 @@ class SessionInfo(gtk.Window):
         set_sound_info(self.microphone_label, None, self.client.microphone_enabled, self.client.sound_source)
 
         self.connection_type_label.set_text(c.info)
-        self.compression_label.set_text(str(p._compression_level))
-        suffix = ""
-        if c.info.lower()=="ssh":
-            suffix = " (%s)" % c.info
-        self.input_encryption_label.set_text((p.cipher_in_name or "None")+suffix)
-        self.output_encryption_label.set_text((p.cipher_out_name or "None")+suffix)
+        protocol_state = p.save_state()
+        level = protocol_state.get("compression_level")
+        if level==0:
+            compression_str = "None"
+        else:
+            compression_str = " + ".join([x for x in ("zlib", "lz4", "bencode", "rencode") if protocol_state.get(x, False)==True])
+            compression_str += ", level %s" % level
+        self.compression_label.set_text(compression_str)
+
+        def enclabel(label, cipher):
+            if not cipher:
+                info = "None"
+            else:
+                info = str(cipher)
+            if c.info.lower()=="ssh":
+                info += " (%s)" % c.info
+            if get_network_caps().get("pycrypto.fastmath", False):
+                info += " (fastmath available)"
+            label.set_text(info)
+        enclabel(self.input_encryption_label, p.cipher_in_name)
+        enclabel(self.output_encryption_label, p.cipher_out_name)
         return True
 
+
+    def getval(self, prefix, suffix, alt=""):
+        if self.client.server_last_info is None:
+            return ""
+        altv = ""
+        if alt:
+            altv = self.client.server_last_info.get(alt+"."+suffix, "")
+        return self.client.server_last_info.get(prefix+"."+suffix, altv)
+
+    def values_from_info(self, prefix, alt=None):
+        def getv(suffix):
+            return self.getval(prefix, suffix, alt)
+        return getv("cur"), getv("min"), getv("avg"), getv("90p"), getv("max")
+
+    def all_values_from_info(self, *window_props):
+        def avg(values):
+            if not values:
+                return ""
+            return sum(values) / len(values)
+        def getv(suffix, op):
+            if self.client.server_last_info is None:
+                return ""
+            values = []
+            for wid in self.client._window_to_id.values():
+                for window_prop in window_props:
+                    v = self.client.server_last_info.get("window[%s].%s.%s" % (wid, window_prop, suffix))
+                    if v is not None:
+                        values.append(v)
+                        break
+            try:
+                return op(values)
+            except:
+                #no values?
+                return ""
+        return getv("cur", avg), getv("min", min), getv("avg", avg), getv("90p", avg), getv("max", max)
+
     def populate_statistics(self):
         log("populate_statistics()")
         if time.time()-self.last_populate_statistics<1.0:
@@ -602,40 +788,10 @@ class SessionInfo(gtk.Window):
             setlabels(self.client_latency_labels, cpl)
         if self.client.windows_enabled:
             if self.client.server_info_request:
-                def values_from_info(prefix, alt=None):
-                    def getv(suffix):
-                        if self.client.server_last_info is None:
-                            return ""
-                        altv = ""
-                        if alt:
-                            altv = self.client.server_last_info.get(alt+"."+suffix, "")
-                        return self.client.server_last_info.get(prefix+"."+suffix, altv)
-                    return getv("cur"), getv("min"), getv("avg"), getv("90p"), getv("max")
-                setall(self.batch_labels, values_from_info("batch_delay", "batch.delay"))
-                setall(self.damage_labels, values_from_info("damage_out_latency", "damage.out_latency"))
-                def all_values_from_info(*window_props):
-                    def avg(values):
-                        if not values:
-                            return ""
-                        return sum(values) / len(values)
-                    def getv(suffix, op):
-                        if self.client.server_last_info is None:
-                            return ""
-                        values = []
-                        for wid in self.client._window_to_id.values():
-                            for window_prop in window_props:
-                                v = self.client.server_last_info.get("window[%s].%s.%s" % (wid, window_prop, suffix))
-                                if v is not None:
-                                    values.append(v)
-                                    break
-                        try:
-                            return op(values)
-                        except:
-                            #no values?
-                            return ""
-                    return getv("cur", avg), getv("min", min), getv("avg", avg), getv("90p", avg), getv("max", max)
-                setall(self.quality_labels, all_values_from_info("quality", "encoding.quality"))
-                setall(self.speed_labels, all_values_from_info("speed", "encoding.speed"))
+                setall(self.batch_labels, self.values_from_info("batch_delay", "batch.delay"))
+                setall(self.damage_labels, self.values_from_info("damage_out_latency", "damage.out_latency"))
+                setall(self.quality_labels, self.all_values_from_info("quality", "encoding.quality"))
+                setall(self.speed_labels, self.all_values_from_info("speed", "encoding.speed"))
 
             region_sizes = []
             rps = []
@@ -682,9 +838,43 @@ class SessionInfo(gtk.Window):
             self.trays_managed_label.set_text(str(trays))
             if self.client.client_supports_opengl:
                 self.opengl_label.set_text(str(gl))
+
+            #remove all the current labels:
+            for x in self.encoder_info_box.get_children():
+                self.encoder_info_box.remove(x)
+            window_encoder_stats = {}
+            if self.client.server_last_info:
+                #We are interested in data like:
+                #window[1].encoder=x264
+                #window[1].encoder.frames=1
+                for k,v in self.client.server_last_info.items():
+                    pos = k.find("].encoder")
+                    if k.startswith("window[") and pos>0:
+                        wid_str = k[len("window["):pos]     #ie: "1"
+                        ekey = k[(pos+len("].encoder")):]   #ie: "" or ".frames"
+                        if ekey.startswith("."):
+                            ekey = ekey[1:]
+                        try:
+                            wid = int(wid_str)
+                            props = window_encoder_stats.setdefault(wid, {})
+                            props[ekey] = v
+                        except:
+                            #wid_str may be invalid, ie:
+                            #window[1].pipeline_option[1].encoder=codec_spec(xpra.codecs.enc_x264.encoder.Encoder)
+                            # -> wid_str= "1].pipeline_option[1"
+                            pass
+                #print("window_encoder_stats=%s" % window_encoder_stats)
+                for wid, props in window_encoder_stats.items():
+                    l = label("%s (%s)" % (wid, props.get("")))
+                    l.show()
+                    info = ["%s=%s" % (k,v) for k,v in props.items() if k!=""]
+                    set_tooltip_text(l, " ".join(info))
+                    self.encoder_info_box.add(l)
         return True
 
     def populate_graphs(self, *args):
+        if self.client.server_info_request:
+            self.client.send_info_request()
         box = self.tab_box
         _, h = box.size_request()
         _, bh = self.tab_button_box.size_request()
@@ -716,12 +906,28 @@ class SessionInfo(gtk.Window):
                                        start_x_offset=start_x_offset)
             self.bandwidth_graph.set_size_request(*pixmap.get_size())
             self.bandwidth_graph.set_from_pixmap(pixmap, None)
+        if self.client.server_info_request:
+            pass
         #latency graph:
-        for l in (self.server_latency, self.client_latency):
+        latency_graph_items = (
+                                (self.avg_ping_latency, "network"),
+                                (self.avg_batch_delay, "batch delay"),
+                                (self.avg_damage_out_latency, "encode&send"),
+                                (self.avg_decoding_latency, "decoding"),
+                                (self.avg_total, "frame total"),
+                                )
+        latency_graph_values = []
+        labels = []
+        for l, name in latency_graph_items:
+            if len(l)==0:
+                continue
+            l = list(l)
             if len(l)<20:
                 for _ in range(20-len(l)):
                     l.insert(0, None)
-        pixmap = make_graph_pixmap([self.server_latency, self.client_latency], labels=["server", "client"],
+            latency_graph_values.append(l)
+            labels.append(name)
+        pixmap = make_graph_pixmap(latency_graph_values, labels=labels,
                                     width=w, height=h/2,
                                     title="Latency (ms)", min_y_scale=10, rounding=25,
                                     start_x_offset=start_x_offset)
