@@ -4,10 +4,20 @@
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
+import time
+
 
 class ImageWrapper(object):
 
-    def __init__(self, x, y, width, height, pixels, pixel_format, depth, rowstride, planes=1):
+    PACKED = 0
+    _3_PLANES = 3
+    _4_PLANES = 4
+    PLANE_OPTIONS = (PACKED, _3_PLANES, _4_PLANES)
+    PLANE_NAMES = {PACKED       : "PACKED",
+                   _3_PLANES    : "3_PLANES",
+                   _4_PLANES    : "4_PLANES"}
+
+    def __init__(self, x, y, width, height, pixels, pixel_format, depth, rowstride, planes=PACKED, thread_safe=True):
         self.x = x
         self.y = y
         self.width = width
@@ -17,10 +27,12 @@ class ImageWrapper(object):
         self.depth = depth
         self.rowstride = rowstride
         self.planes = planes
+        self.thread_safe = thread_safe
         self.freed = False
+        self.timestamp = int(time.time()*1000)
 
-    def __str__(self):
-        return "%s(%s:%s)" % (type(self), self.pixel_format, self.get_geometry())
+    def __repr__(self):
+        return "%s(%s:%s:%s)" % (type(self), self.pixel_format, self.get_geometry(), ImageWrapper.PLANE_NAMES.get(self.planes))
 
     def get_geometry(self):
         return self.x, self.y, self.width, self.height, self.depth
@@ -55,6 +67,20 @@ class ImageWrapper(object):
     def get_planes(self):
         return self.planes
 
+    def is_thread_safe(self):
+        """ if True, free() and clone_pixel_data() can be called from any thread,
+            if False, free() and clone_pixel_data() must be called from the same thread.
+            Used by XImageWrapper to ensure X11 images are freed from the UI thread.
+        """
+        return self.thread_safe
+
+    def get_timestamp(self):
+        """ time in millis """
+        return self.timestamp
+
+
+    def set_timestamp(self, timestamp):
+        self.timestamp = timestamp
 
     def set_planes(self, planes):
         self.planes = planes
@@ -78,6 +104,7 @@ class ImageWrapper(object):
                 assert self.planes>0
                 for i in range(self.planes):
                     self.pixels[i] = self.pixels[i][:]
+            self.thread_safe = True
 
     def __del__(self):
         #print("ImageWrapper.__del__() calling %s" % self.free)
