@@ -1,10 +1,10 @@
 # This file is part of Xpra.
-# Copyright (C) 2011-2013 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2014 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 from xpra.log import Logger
-log = Logger()
+log = Logger("xor")
 
 try:
     from xpra.codecs.xor.cyxor import xor_str  #@UnresolvedImport
