@@ -18,13 +18,18 @@ o0700 = 448     #0o700
 class ServerSockInUse(Exception):
     pass
 
+def osexpand(s, actual_username=""):
+    if len(actual_username)>0 and s.startswith("~/"):
+        #replace "~/" with "~$actual_username/"
+        s = "~%s/%s" % (actual_username, s[2:])
+    return os.path.expandvars(os.path.expanduser(s))
+
+
 class DotXpra(object):
-    def __init__(self, sockdir=None, confdir=None):
+    def __init__(self, sockdir=None, confdir=None, actual_username=""):
         from xpra.platform.paths import get_default_socket_dir, get_default_conf_dir
-        def expand(s):
-            return os.path.expandvars(os.path.expanduser(s))
-        self._confdir = expand(confdir or get_default_conf_dir())
-        self._sockdir = expand(sockdir or get_default_socket_dir())
+        self._confdir = osexpand(confdir or get_default_conf_dir(), actual_username)
+        self._sockdir = osexpand(sockdir or get_default_socket_dir(), actual_username)
         if not os.path.exists(self._confdir):
             os.mkdir(self._confdir, o0700)
         if not os.path.exists(self._sockdir):
@@ -37,7 +42,7 @@ class DotXpra(object):
     def confdir(self):
         return self._confdir
 
-    def _normalize_local_display_name(self, local_display_name):
+    def normalize_local_display_name(self, local_display_name):
         if not local_display_name.startswith(":"):
             local_display_name = ":" + local_display_name
         if "." in local_display_name:
@@ -47,27 +52,29 @@ class DotXpra(object):
             assert char in "0123456789", "invalid character in display name: %s" % char
         return local_display_name
 
-    def make_path(self, local_display_name, dirpath ):
-        local_display_name = self._normalize_local_display_name(local_display_name)
-        return os.path.join( dirpath , self._prefix + local_display_name[1:])
+    def norm_make_path(self, name, dirpath):
+        return os.path.join(dirpath, self._prefix + name)
 
     def socket_path(self, local_display_name):
-        return self.make_path(local_display_name, self._sockdir)
+        return self.norm_make_path(local_display_name[1:], self._sockdir)
 
     def log_path(self, local_display_name):
-        return self.make_path(local_display_name, self._confdir)
+        return self.norm_make_path(local_display_name[1:], self._confdir)
 
     LIVE = "LIVE"
     DEAD = "DEAD"
     UNKNOWN = "UNKNOWN"
     def server_state(self, local_display_name, timeout=5):
-        path = self.socket_path(local_display_name)
-        if not os.path.exists(path):
+        socket_path = self.socket_path(local_display_name)
+        return self.get_server_state(socket_path, timeout)
+
+    def get_server_state(self, socket_path, timeout=5):
+        if not os.path.exists(socket_path):
             return self.DEAD
         sock = socket.socket(socket.AF_UNIX)
         sock.settimeout(timeout)
         try:
-            sock.connect(path)
+            sock.connect(socket_path)
         except socket.error, e:
             err = e.args[0]
             if err==errno.ECONNREFUSED:
@@ -103,13 +110,18 @@ class DotXpra(object):
             os.unlink(socket_path)
         return socket_path
 
-    def sockets(self):
+    def sockets(self, check_uid=0):
         results = []
         base = os.path.join(self._sockdir, self._prefix)
         potential_sockets = glob.glob(base + "*")
-        for path in potential_sockets:
-            if stat.S_ISSOCK(os.stat(path).st_mode):
-                local_display = ":" + path[len(base):]
-                state = self.server_state(local_display)
+        for path in sorted(potential_sockets):
+            s = os.stat(path)
+            if stat.S_ISSOCK(s.st_mode):
+                if check_uid>0:
+                    if s.st_uid!=check_uid:
+                        #socket uid does not match
+                        continue
+                state = self.get_server_state(path)
+                local_display = ":"+path[len(base):]
                 results.append((state, local_display))
         return results
