@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # coding=utf8
 # This file is part of Xpra.
-# Copyright (C) 2013 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2013, 2014 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -22,19 +22,27 @@ except ImportError:
     from Queue import Queue             #@Reimport @UnusedImport
 
 
-
-SIGNAMES = {signal.SIGINT:"SIGINT", signal.SIGTERM:"SIGTERM"}
+SIGNAMES = {}
+for x in [x for x in dir(signal) if x.startswith("SIG")]:
+    SIGNAMES[getattr(signal, x)] = x
 
 
-#python3 making life difficult:
+#use cStringIO, fallback to StringIO,
+#and python3 is making life more difficult yet again:
 try:
-    from io import BytesIO as BytesIOClass          #@UnusedImport
+    from io import BytesIO as BytesIOClass              #@UnusedImport
 except:
-    from StringIO import StringIO as BytesIOClass   #@Reimport @UnusedImport
+    try:
+        from cStringIO import StringIO as BytesIOClass  #@Reimport @UnusedImport
+    except:
+        from StringIO import StringIO as BytesIOClass   #@Reimport @UnusedImport
 try:
-    from StringIO import StringIO as StringIOClass  #@UnusedImport
+    from StringIO import StringIO as StringIOClass      #@UnusedImport
 except:
-    from io import StringIO as StringIOClass        #@Reimport @UnusedImport
+    try:
+        from cStringIO import StringIO as StringIOClass #@Reimport @UnusedImport
+    except:
+        from io import StringIO as StringIOClass        #@Reimport @UnusedImport
 
 
 if sys.version < '3':
@@ -82,30 +90,13 @@ def platform_name(sys_platform, release):
             return rel(v)
     return rel(sys_platform)
 
-def set_prgname(name):
-    try:
-        import glib
-        glib.set_prgname(name)
-    except:
-        pass
-
-
-NAME_SET = False
-def set_application_name(name):
-    global NAME_SET
-    if NAME_SET:
-        return
-    NAME_SET = True
-    from xpra.log import Logger
-    log = Logger()
-    if sys.version_info[:2]<(2,5):
-        log.warn("Python %s is too old!", sys.version_info)
-        return
-    try:
-        import glib
-        glib.set_application_name(name or "Xpra")
-    except ImportError, e:
-        log.warn("glib is missing, cannot set the application name, please install glib's python bindings: %s", e)
+def os_info(sys_platform, platform_release, platform_platform, platform_linux_distribution):
+    s = [platform_name(sys_platform, platform_release)]
+    if platform_linux_distribution and len(platform_linux_distribution)==3 and len(platform_linux_distribution[0])>0:
+        s.append(" ".join(platform_linux_distribution))
+    elif platform_platform:
+        s.append(platform_platform)
+    return s
 
 
 try:
@@ -134,29 +125,60 @@ except ImportError:
 
 
 def get_machine_id():
+    """
+        Try to get uuid string which uniquely identifies this machine.
+        Warning: only works on posix!
+        (which is ok since we only used it on posix at present)
+    """
     v = u""
-    for filename in ["/etc/machine-id", "/var/lib/dbus/machine-id"]:
-        if os.path.exists(filename) and os.path.isfile(filename):
-            f = None
-            try:
-                try:
-                    f = open(filename, 'rb')
-                    v = f.read()
-                    break
-                finally:
-                    if f:
-                        f.close()
-            except Exception:
-                pass
+    if os.name=="posix":
+        for filename in ["/etc/machine-id", "/var/lib/dbus/machine-id"]:
+            v = load_binary_file(filename)
+            if v is not None:
+                break
     return  str(v).strip("\n\r")
 
+def get_user_uuid():
+    """
+        Try to generate a uuid string which is unique to this user.
+        (relies on get_machine_id to uniquely identify a machine)
+    """
+    try:
+        import hashlib
+        u = hashlib.sha1()
+    except:
+        #try python2.4 variant:
+        import sha
+        u = sha.new()
+    def uupdate(ustr):
+        u.update(ustr.encode("utf-8"))
+    uupdate(get_machine_id())
+    if os.name=="posix":
+        uupdate(u"/")
+        uupdate(str(os.getuid()))
+        uupdate(u"/")
+        uupdate(str(os.getgid()))
+    uupdate(os.environ.get("HOME", ""))
+    return u.hexdigest()
+
+
+def load_binary_file(filename):
+    if not os.path.exists(filename):
+        return None
+    f = None
+    try:
+        f = open(filename, "rb")
+        try:
+            return f.read()
+        finally:
+            f.close()
+    except:
+        return None
+
 
 def main():
-    import logging
-    logging.basicConfig(format="%(asctime)s %(message)s")
-    logging.root.setLevel(logging.INFO)
     from xpra.log import Logger
-    log = Logger("")
+    log = Logger("util")
     sp = sys.platform
     log.info("platform_name(%s)=%s", sp, platform_name(sp, ""))
     log.info("get_machine_id()=%s", get_machine_id())
