@@ -1,11 +1,11 @@
 # coding=utf8
 # This file is part of Xpra.
-# Copyright (C) 2013 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2013, 2014 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 from xpra.log import Logger
-log = Logger()
+log = Logger("shadow", "osx")
 
 from xpra.server.gtk_server_base import GTKServerBase
 from xpra.server.shadow_server_base import ShadowServerBase, RootWindowModel
@@ -74,8 +74,8 @@ class ShadowServer(ShadowServerBase, GTKServerBase):
         ShadowServerBase.__init__(self, gtk.gdk.get_default_root_window())
         GTKServerBase.__init__(self)
 
-    def init(self, sockets, opts):
-        GTKServerBase.init(self, sockets, opts)
+    def init(self, opts):
+        GTKServerBase.init(self, opts)
         self.keycodes = {}
 
     def makeRootWindowModel(self):
