@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2010 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2011-2013 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2014 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -61,7 +61,7 @@ def get_icon_filename(name):
     def err(*msg):
         """ log an error message and return None """
         from xpra.log import Logger
-        log = Logger()
+        log = Logger("icon")
         log.error(*msg)
         return None
     idir = get_icon_dir()
@@ -104,6 +104,7 @@ def get_license_text(self):
 
 from xpra.platform import platform_import
 platform_import(globals(), "paths", True,
+                "get_resources_dir",
                 "get_app_dir",
                 "get_icon_dir")
 platform_import(globals(), "paths", False,
