@@ -1,15 +1,15 @@
 # coding=utf8
 # This file is part of Xpra.
-# Copyright (C) 2012-2013 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2012-2014 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import win32api         #@UnresolvedImport
 import win32con         #@UnresolvedImport
 from xpra.log import Logger
-log = Logger()
+log = Logger("shadow", "win32")
 
-from xpra.server.gtk_shadow_server_base import GTKRootWindowModel
+from xpra.server.gtk_root_window_model import GTKRootWindowModel
 from xpra.server.gtk_server_base import GTKServerBase
 from xpra.server.shadow_server_base import ShadowServerBase
 
