@@ -1,20 +1,21 @@
 # This file is part of Xpra.
 # Copyright (C) 2010 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2011-2013 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2014 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 # Ubuntu re-invents the wheel, and it's a broken one
 
 import os
-from xpra.client.tray_base import TrayBase, debug
+
+from xpra.log import Logger
+log = Logger("tray", "posix")
+
+from xpra.util import is_unity
+from xpra.client.tray_base import TrayBase
 from xpra.platform.paths import get_icon_dir
 
 
-def is_unity():
-    return os.environ.get("XDG_CURRENT_DESKTOP", "").lower() == "unity"
-
-
 _appindicator = False
 def get_appindicator():
     global _appindicator
@@ -40,21 +41,22 @@ def can_use_appindicator():
 
 class AppindicatorTray(TrayBase):
 
-    def __init__(self, menu, tooltip, icon_filename, size_changed_cb, click_cb, mouseover_cb, exit_cb):
-        TrayBase.__init__(self, menu, tooltip, icon_filename, size_changed_cb, click_cb, mouseover_cb, exit_cb)
-        filename = self.get_tray_icon_filename(icon_filename)
+    def __init__(self, *args):
+        TrayBase.__init__(self, *args)
+        filename = self.get_tray_icon_filename(self.default_icon_filename)
         self.appindicator = get_appindicator()
+        self._has_icon = False
         assert self.appindicator, "appindicator is not available!"
-        self.tray_widget = self.appindicator.Indicator(tooltip, filename, self.appindicator.CATEGORY_APPLICATION_STATUS)
+        self.tray_widget = self.appindicator.Indicator(self.tooltip, filename, self.appindicator.CATEGORY_APPLICATION_STATUS)
         if hasattr(self.tray_widget, "set_icon_theme_path"):
             self.tray_widget.set_icon_theme_path(get_icon_dir())
         self.tray_widget.set_attention_icon("xpra.png")
         if filename:
-            self.tray_widget.set_icon(filename)
-        else:
+            self.set_icon_from_file(filename)
+        if not self._has_icon:
             self.tray_widget.set_label("Xpra")
-        if menu:
-            self.tray_widget.set_menu(menu)
+        if self.menu:
+            self.tray_widget.set_menu(self.menu)
 
     def hide(self, *args):
         self.tray_widget.set_status(self.appindicator.STATUS_PASSIVE)
@@ -67,38 +69,56 @@ class AppindicatorTray(TrayBase):
         pass
 
     def set_tooltip(self, text=None):
-        self.tray_widget.set_label(text or "Xpra")
+        #we only use this if we haven't got an icon
+        #as with appindicator this creates a large text label
+        #next to where the icon is/should be
+        if not self._has_icon:
+            self.tray_widget.set_label(text or "Xpra")
 
     def set_icon_from_data(self, pixels, has_alpha, w, h, rowstride):
-        #can't do that either..
-        pass
+        #use a temporary file (yuk)
+        try:
+            from gtk import gdk
+        except:
+            #no gtk.gdk... no can do
+            return
+        import tempfile
+        try:
+            _, filename = tempfile.mkstemp(suffix="png")
+            log("set_icon_from_data%s using temporary file %s", ("%s pixels" % len(pixels), has_alpha, w, h, rowstride), filename)
+            tray_icon = gdk.pixbuf_new_from_data(pixels, gdk.COLORSPACE_RGB, has_alpha, 8, w, h, rowstride)
+            tray_icon.save(filename, "png")
+            self.do_set_icon_from_file(filename)
+        finally:
+            os.unlink(filename)
 
     def do_set_icon_from_file(self, filename):
         if not hasattr(self.tray_widget, "set_icon_theme_path"):
             self.tray_widget.set_icon(filename)
+            self._has_icon = True
             return
         head, icon_name = os.path.split(filename)
         if head:
+            log("do_set_icon_from_file(%s) setting icon theme path=%s", filename, head)
             self.tray_widget.set_icon_theme_path(head)
         #remove extension (wtf?)
         dot = icon_name.rfind(".")
         if dot>0:
             icon_name = icon_name[:dot]
+        log("do_set_icon_from_file(%s) setting icon=%s", filename, icon_name)
         self.tray_widget.set_icon(icon_name)
+        self._has_icon = True
 
 
 def main():
-    import logging
-    logging.basicConfig(format="%(asctime)s %(message)s")
-    logging.root.setLevel(logging.DEBUG)
-
+    log.enable_debug()
     appindicator = get_appindicator()
     if not appindicator:
-        debug("appindicator not available")
+        log("appindicator not available")
         return
 
     if not can_use_appindicator():
-        debug("appindicator may not be shown...")
+        log("appindicator may not be shown...")
 
     from xpra.gtk_common.gobject_compat import import_gobject, import_gtk
     gobject = import_gobject()
