@@ -16,7 +16,7 @@ SHADOW_SUPPORTED = True
 
 
 #don't bother trying to forward system tray with Ubuntu's "unity":
-from xpra.platform.xposix.appindicator_tray import is_unity
+from xpra.util import is_unity
 SYSTEM_TRAY_SUPPORTED = not is_unity()
 MMAP_SUPPORTED = True
 CAN_DAEMONIZE = True
