@@ -1,5 +1,5 @@
 # This file is part of Xpra.
-# Copyright (C) 2013 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2013, 2014 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -28,6 +28,6 @@ def get_sys_info():
                 info[prefix+var] = value
     except:
         from xpra.log import Logger
-        log = Logger()
+        log = Logger("posix")
         log.error("error getting memory usage info", exc_info=True)
     return info
