@@ -15,128 +15,6 @@ from xpra.util import AdHocStruct
 def warn(msg):
     sys.stderr.write(msg+"\n")
 
-def codec_import_check(name, top_module, class_module, *classnames):
-    try:
-        try:
-            __import__(top_module, {}, {}, [])
-            for classname in classnames:
-                return __import__(class_module, {}, {}, classname)
-        except ImportError, e:
-            #the required module does not exist
-            #xpra was probably built with the option: --without-${name}
-            pass
-    except Exception, e:
-        warn("cannot load %s: %s missing from %s: %s" % (name, classname, class_module, e))
-    return None
-codec_versions = {}
-def add_codec_version(name, top_module, fieldname, invoke=False):
-    try:
-        module = __import__(top_module, {}, {}, [fieldname])
-        if not hasattr(module, fieldname):
-            warn("cannot find %s in %s" % (fieldname, module))
-            return
-        v = getattr(module, fieldname)
-        if invoke and v:
-            v = v()
-        global codec_versions
-        codec_versions[name] = v
-    except ImportError:
-        #not present
-        pass
-
-PIL = codec_import_check("Python Imaging Library", "PIL", "PIL", "Image")
-has_PIL = PIL is not None
-add_codec_version("PIL", "PIL.Image", "VERSION")
-
-enc_vpx = codec_import_check("vpx encoder", "xpra.codecs.vpx", "xpra.codecs.vpx.encoder", "Encoder")
-has_enc_vpx = enc_vpx is not None
-dec_vpx = codec_import_check("vpx decoder", "xpra.codecs.vpx", "xpra.codecs.vpx.decoder", "Decoder")
-has_dec_vpx = dec_vpx is not None
-add_codec_version("vpx", "xpra.codecs.vpx.encoder", "get_version", True)
-
-enc_x264 = codec_import_check("x264 encoder", "xpra.codecs.enc_x264", "xpra.codecs.enc_x264.encoder", "Encoder")
-has_enc_x264 = enc_x264 is not None
-add_codec_version("x264", "xpra.codecs.enc_x264.encoder", "get_version", True)
-
-enc_nvenc = codec_import_check("nvenc encoder", "xpra.codecs.nvenc", "xpra.codecs.nvenc.encoder", "Encoder")
-has_enc_nvenc = enc_nvenc is not None
-add_codec_version("nvenc", "xpra.codecs.nvenc.encoder", "get_version", True)
-
-csc_swscale = codec_import_check("csc swscale", "xpra.codecs.csc_swscale", "xpra.codecs.csc_swscale.colorspace_converter", "ColorspaceConverter")
-has_csc_swscale = csc_swscale is not None
-add_codec_version("swscale", "xpra.codecs.csc_swscale.colorspace_converter", "get_version", True)
-
-csc_nvcuda = None   #codec_import_check("csc nvcuda", "xpra.codecs.csc_nvcuda", "xpra.codecs.csc_nvcuda.colorspace_converter", "ColorspaceConverter")
-has_csc_nvcuda = csc_nvcuda is not None
-add_codec_version("nvcuda", "xpra.codecs.csc_nvcuda.colorspace_converter", "get_version", True)
-
-dec_avcodec = codec_import_check("avcodec decoder", "xpra.codecs.dec_avcodec", "xpra.codecs.dec_avcodec.decoder", "Decoder")
-if dec_avcodec is not None:
-    add_codec_version("avcodec", "xpra.codecs.dec_avcodec.decoder", "get_version", True)
-else:
-    dec_avcodec = codec_import_check("avcodec decoder", "xpra.codecs.dec_avcodec2", "xpra.codecs.dec_avcodec2.decoder", "Decoder")
-    add_codec_version("avcodec", "xpra.codecs.dec_avcodec2.decoder", "get_version", True)
-has_dec_avcodec = dec_avcodec is not None
-
-enc_webp, enc_webp, has_enc_webp_lossless, webp_handlers = None, None, False, None
-try:
-    #python >=2.6 only and required for webp to work:
-    bytearray()
-    try:
-        #these symbols are all available upstream as of libwebp 0.2:
-        enc_webp = codec_import_check("webp encoder", "xpra.codecs.webm", "xpra.codecs.webm.encode", "EncodeRGB", "EncodeRGBA", "EncodeBGR", "EncodeBGRA")
-        dec_webp = codec_import_check("webp encoder", "xpra.codecs.webm", "xpra.codecs.webm.decode", "DecodeRGB", "DecodeRGBA", "DecodeBGR", "DecodeBGRA")
-        #these symbols were added in libwebp 0.4, and we added HAS_LOSSLESS to the wrapper:
-        _enc_webp_lossless = codec_import_check("webp encoder", "xpra.codecs.webm", "xpra.codecs.webm.encode", "HAS_LOSSLESS", "EncodeLosslessRGB", "EncodeLosslessRGBA", "EncodeLosslessBGRA", "EncodeLosslessBGR")
-        if _enc_webp_lossless:
-            #the fact that the python functions are defined is not enough
-            #we need to check if the underlying C functions actually exist:
-            has_enc_webp_lossless = _enc_webp_lossless.HAS_LOSSLESS
-        add_codec_version("webp", "xpra.codecs.webm", "__VERSION__")
-        webp_handlers = codec_import_check("webp bitmap handler", "xpra.codecs.webm", "xpra.codecs.webm.handlers", "BitmapHandler")
-    except Exception, e:
-        warn("cannot load webp: " % e)
-except:
-    #no bytearray, no webp
-    pass
-has_enc_webp = enc_webp is not None
-has_dec_webp = dec_webp is not None
-
-
-PREFERED_ENCODING_ORDER = ["x264", "vpx", "webp", "png", "png/P", "png/L", "rgb", "jpeg"]
-
-ENCODINGS_TO_NAME = {
-                  "x264"    : "H.264",
-                  "vpx"     : "VPx",
-                  "png"     : "PNG (24/32bpp)",
-                  "png/P"   : "PNG (8bpp colour)",
-                  "png/L"   : "PNG (8bpp grayscale)",
-                  "webp"    : "WebP",
-                  "jpeg"    : "JPEG",
-                  "rgb"     : "Raw RGB + zlib (24/32bpp)",
-                }
-
-ENCODINGS_HELP = {
-                  "x264"    : "H.264 video codec",
-                  "vpx"     : "VPx video codec",
-                  "png"     : "Portable Network Graphics (24 or 32bpp for transparency)",
-                  "png/P"   : "Portable Network Graphics (8bpp colour)",
-                  "png/L"   : "Portable Network Graphics (8bpp grayscale)",
-                  "webp"    : "WebP compression (lossless or lossy)",
-                  "jpeg"    : "JPEG lossy compression",
-                  "rgb"     : "Raw RGB pixels, lossless, compressed using zlib (24 or 32bpp for transparency)",
-                  }
-
-HELP_ORDER = ("x264", "vpx", "webp", "png", "png/P", "png/L", "rgb", "jpeg")
-
-def encodings_help(encodings):
-    h = []
-    for e in HELP_ORDER:
-        if e in encodings:
-            ehelp = ENCODINGS_HELP.get(e)
-            h.append(e.ljust(12) + ehelp)
-    return h
-
 
 ENCRYPTION_CIPHERS = []
 try:
@@ -146,10 +24,18 @@ try:
 except:
     pass
 
+try:
+    import xpra.sound
+    has_sound_support = bool(xpra.sound)
+except:
+    has_sound_support = False
+
 
 def OpenGL_safety_check():
     #Ubuntu 12.04 will just crash on you if you try:
-    distro = python_platform.linux_distribution()
+    distro = ""
+    if hasattr(python_platform, "linux_distribution"):
+        distro = python_platform.linux_distribution()
     if distro and len(distro)==3 and distro[0]=="Ubuntu":
         ur = distro[1]  #ie: "12.04"
         try:
@@ -188,57 +74,6 @@ if OpenGL_safety_check() is not None:
 
 
 
-# we end up initializing gstreamer here and it does things
-# we don't want with sys.argv, so hack around it:
-saved_args = sys.argv
-sys.argv = sys.argv[:1]
-try:
-    from xpra.sound import gstreamer_util   #@UnusedImport
-    HAS_SOUND = True
-except:
-    HAS_SOUND = False
-sys.argv = saved_args
-
-def get_codecs(is_speaker, is_server):
-    if not HAS_SOUND:
-        return []
-    try:
-        from xpra.sound.gstreamer_util import can_encode, can_decode
-        if (is_server and is_speaker) or (not is_server and not is_speaker):
-            return can_encode()
-        else:
-            return can_decode()
-    except Exception, e:
-        warn("failed to get list of codecs: %s" % e)
-        return []
-
-def show_codec_help(is_server, speaker_codecs, microphone_codecs):
-    all_speaker_codecs = get_codecs(True, is_server)
-    invalid_sc = [x for x in speaker_codecs if x not in all_speaker_codecs]
-    hs = "help" in speaker_codecs
-    if hs:
-        print("speaker codecs available: %s" % (", ".join(all_speaker_codecs)))
-    elif len(invalid_sc):
-        warn("WARNING: some of the specified speaker codecs are not available: %s" % (", ".join(invalid_sc)))
-        for x in invalid_sc:
-            speaker_codecs.remove(x)
-    elif len(speaker_codecs)==0:
-        speaker_codecs += all_speaker_codecs
-
-    all_microphone_codecs = get_codecs(True, is_server)
-    invalid_mc = [x for x in microphone_codecs if x not in all_microphone_codecs]
-    hm = "help" in microphone_codecs
-    if hm:
-        print("microphone codecs available: %s" % (", ".join(all_microphone_codecs)))
-    elif len(invalid_mc):
-        warn("WARNING: some of the specified microphone codecs are not available: %s" % (", ".join(invalid_mc)))
-        for x in invalid_mc:
-            microphone_codecs.remove(x)
-    elif len(microphone_codecs)==0:
-        microphone_codecs += all_microphone_codecs
-    return hm or hs
-
-
 def get_build_info():
     info = []
     try:
@@ -266,10 +101,12 @@ def get_build_info():
     return info
 
 
-def save_config(conf_file, config, keys):
+def save_config(conf_file, config, keys, extras={}):
     f = open(conf_file, "w")
+    option_types = OPTION_TYPES.copy()
+    option_types.update(extras)
     for key in keys:
-        assert key in OPTION_TYPES, "invalid configuration key: %s" % key
+        assert key in option_types, "invalid configuration key: %s" % key
         fn = key.replace("-", "_")
         v = getattr(config, fn)
         f.write("%s=%s%s" % (key, v, os.linesep))
@@ -346,18 +183,25 @@ def read_xpra_defaults():
         returns a dict with values as strings and arrays of strings.
     """
     #first, read the global defaults:
-    if sys.platform.startswith("win"):
-        conf_dir = os.path.dirname(os.path.abspath(sys.executable))
+    from xpra.platform.paths import get_resources_dir, get_default_conf_dir
+    if sys.platform.startswith("win") or sys.platform.startswith("darwin"):
+        #OSX and win32 use binary installers,
+        #we must look for the default config in the bundled resource location:
+        conf_dir = get_resources_dir()
     elif sys.prefix == '/usr':
+        #default posix config location:
         conf_dir = '/etc/xpra'
     else:
+        #hope the prefix is something like "/usr/local":
         conf_dir = sys.prefix + '/etc/xpra/'
-    defaults = read_xpra_conf(conf_dir)
+    defaults = {}
+    if conf_dir:
+        defaults = read_xpra_conf(conf_dir)
     #now load the per-user config over it:
-    from xpra.platform.paths import get_default_conf_dir
-    user_defaults = read_xpra_conf(get_default_conf_dir())
-    for k,v in user_defaults.items():
-        defaults[k] = v
+    def_dir = get_default_conf_dir()
+    if def_dir and def_dir!=conf_dir:
+        user_defaults = read_xpra_conf(get_default_conf_dir())
+        defaults.update(user_defaults)
     return defaults
 
 
@@ -367,6 +211,7 @@ OPTION_TYPES = {
                     "title"             : str,
                     "host"              : str,
                     "username"          : str,
+                    "auth"              : str,
                     "remote-xpra"       : str,
                     "session-name"      : str,
                     "client-toolkit"    : str,
@@ -378,13 +223,17 @@ OPTION_TYPES = {
                     "clipboard-filter-file" : str,
                     "pulseaudio-command": str,
                     "encryption"        : str,
+                    "encryption_keyfile": str,
                     "mode"              : str,
                     "ssh"               : str,
                     "xvfb"              : str,
                     "socket-dir"        : str,
                     "log-file"          : str,
                     "mode"              : str,
+                    "border"            : str,
                     "window-layout"     : str,
+                    "display"           : str,
+                    "tcp-proxy"         : str,
                     #int options:
                     "quality"           : int,
                     "min-quality"       : int,
@@ -400,9 +249,11 @@ OPTION_TYPES = {
                     "debug"             : bool,
                     "daemon"            : bool,
                     "use-display"       : bool,
-                    "no-tray"           : bool,
+                    "fake-xinerama"     : bool,
+                    "tray"              : bool,
                     "clipboard"         : bool,
                     "pulseaudio"        : bool,
+                    "dbus-proxy"        : bool,
                     "mmap"              : bool,
                     "mmap-group"        : bool,
                     "speaker"           : bool,
@@ -413,14 +264,20 @@ OPTION_TYPES = {
                     "cursors"           : bool,
                     "bell"              : bool,
                     "notifications"     : bool,
+                    "xsettings"         : bool,
                     "system-tray"       : bool,
                     "sharing"           : bool,
                     "delay-tray"        : bool,
                     "windows"           : bool,
                     "autoconnect"       : bool,
                     "exit-with-children": bool,
+                    "exit-with-client"  : bool,
+                    "exit-ssh"          : bool,
                     "opengl"            : bool,
+                    "mdns"              : bool,
                     #arrays of strings (default value, allowed options):
+                    "video-encoders"    : list,
+                    "csc-modules"       : list,
                     "speaker-codec"     : list,
                     "microphone-codec"  : list,
                     "key-shortcut"      : list,
@@ -437,8 +294,8 @@ def get_defaults():
         return GLOBAL_DEFAULTS
     from xpra.platform.features import DEFAULT_SSH_CMD
     try:
-        import getpass
-        username = getpass.getuser()
+        from xpra.platform.info import get_username
+        username = get_username()
     except:
         username = ""
     GLOBAL_DEFAULTS = {
@@ -446,6 +303,7 @@ def get_defaults():
                     "title"             : "@title@ on @client-machine@",
                     "host"              : "",
                     "username"          : username,
+                    "auth"              : "",
                     "remote-xpra"       : ".xpra/run-xpra",
                     "session-name"      : "",
                     "client-toolkit"    : "",
@@ -460,47 +318,59 @@ def get_defaults():
                                             +" --load=module-null-sink --load=module-native-protocol-unix "
                                             +" --log-level=2 --log-target=stderr",
                     "encryption"        : "",
+                    "encryption_keyfile": "",
                     "mode"              : "tcp",
                     "ssh"               : DEFAULT_SSH_CMD,
                     "xvfb"              : "Xvfb +extension Composite -screen 0 3840x2560x24+32 -nolisten tcp -noreset -auth $XAUTHORITY",
                     "socket-dir"        : "",
                     "log-file"          : "$DISPLAY.log",
+                    "border"            : "auto,0",
                     "window-layout"     : "",
+                    "display"           : "",
+                    "tcp-proxy"         : "",
                     "quality"           : -1,
                     "min-quality"       : 50,
                     "speed"             : -1,
                     "min-speed"         : -1,
                     "port"              : -1,
-                    "compression_level" : 3,
+                    "compression_level" : 1,
                     "dpi"               : 96,
                     "max-bandwidth"     : 0.0,
                     "auto-refresh-delay": 0.25,
                     "debug"             : False,
                     "daemon"            : True,
                     "use-display"       : False,
-                    "no-tray"           : False,
+                    "fake-xinerama"     : True,
+                    "tray"              : True,
                     "clipboard"         : True,
                     "pulseaudio"        : True,
+                    "dbus-proxy"        : os.name=="posix" and not sys.platform.startswith("darwin"),
                     "mmap"              : True,
                     "mmap-group"        : False,
-                    "speaker"           : True,
-                    "microphone"        : True,
+                    "speaker"           : has_sound_support,
+                    "microphone"        : has_sound_support,
                     "readonly"          : False,
                     "keyboard-sync"     : True,
                     "pings"             : False,
                     "cursors"           : True,
                     "bell"              : True,
                     "notifications"     : True,
+                    "xsettings"         : os.name=="posix",
                     "system-tray"       : True,
                     "sharing"           : False,
                     "delay-tray"        : False,
                     "windows"           : True,
                     "autoconnect"       : False,
                     "exit-with-children": False,
+                    "exit-with-client"  : False,
+                    "exit-ssh"          : True,
                     "opengl"            : OPENGL_DEFAULT,
+                    "mdns"              : True,
+                    "video-encoders"    : [],
+                    "csc-modules"       : [],
                     "speaker-codec"     : [],
                     "microphone-codec"  : [],
-                    "key-shortcut"      : ["Meta+Shift+F4:quit", "Meta+Shift+F8:magic_key"],
+                    "key-shortcut"      : ["Meta+Shift+F4:quit", "Meta+Shift+F8:magic_key", "Meta+Shift+F11:show_session_info"],
                     "bind-tcp"          : None,
                     "start-child"       : None,
                     }
@@ -517,9 +387,6 @@ OPTIONS_VALIDATION = {
 CLONES = {
             "quality"       : "jpeg-quality",
           }
-#TODO:
-#"speaker-codec"     : [""],
-#"microphone-codec"  : [""],
 
 #these options should not be specified in config files:
 NO_FILE_OPTIONS = ["daemon"]
@@ -539,11 +406,13 @@ def parse_bool(k, v):
     else:
         warn("Warning: cannot parse value '%s' for '%s' as a boolean" % (v, k))
 
-def print_bool(k, v):
+def print_bool(k, v, true_str='yes', false_str='no'):
     if type(v)==type(None):
         return 'auto'
     if type(v)==bool:
-        return v and 'yes' or 'no'
+        if v:
+            return true_str
+        return false_str
     warn("Warning: cannot print value '%s' for '%s' as a boolean" % (v, k))
 
 def parse_number(numtype, k, v, auto=-1):
@@ -557,19 +426,21 @@ def parse_number(numtype, k, v, auto=-1):
         warn("Warning: cannot parse value '%s' for '%s' as a type %s: %s" % (v, k, numtype, e))
         return None
 
-def validate_config(d={}, discard=NO_FILE_OPTIONS):
+def validate_config(d={}, discard=NO_FILE_OPTIONS, extras={}):
     """
         Validates all the options given in a dict with fields as keys and
         strings or arrays of strings as values.
         Each option is strongly typed and invalid value are discarded.
         We get the required datatype from OPTION_TYPES
     """
+    option_types = OPTION_TYPES.copy()
+    option_types.update(extras)
     nd = {}
     for k, v in d.items():
         if k in discard:
             warn("Warning: option '%s' is not allowed in configuration files" % k)
             continue
-        vt = OPTION_TYPES.get(k)
+        vt = option_types.get(k)
         if vt is None:
             warn("Warning: invalid option: '%s'" % k)
             continue
