@@ -1,7 +1,7 @@
 # coding=utf8
 # This file is part of Xpra.
 # Copyright (C) 2011 Serviware (Arthur Huillet, <ahuillet@serviware.com>)
-# Copyright (C) 2010-2013 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2014 Antoine Martin <antoine@devloop.org.uk>
 # Copyright (C) 2008 Nathaniel Smith <njs@pobox.com>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
@@ -9,18 +9,15 @@
 import gtk.gdk
 import gobject
 gobject.threads_init()
-gtk.gdk.threads_init()
 
 from xpra.log import Logger
-log = Logger()
+log = Logger("server", "gtk")
 
 from xpra.gtk_common.quit import (gtk_main_quit_really,
                            gtk_main_quit_on_fatal_exceptions_enable)
 from xpra.server.server_base import ServerBase
 from xpra.gtk_common.gtk_util import add_gtk_version_info
 from xpra.gtk_common.gtk2common import gtk2main
-from xpra.net.protocol import set_scheduler
-set_scheduler(gobject)
 
 
 class GTKServerBase(ServerBase):
@@ -59,16 +56,17 @@ class GTKServerBase(ServerBase):
         capabilities = ServerBase.make_hello(self)
         capabilities["display"] = gtk.gdk.display_get_default().get_name()
         capabilities["cursor.default_size"] = gtk.gdk.display_get_default().get_default_cursor_size()
+        capabilities["cursor.max_size"] = gtk.gdk.display_get_default().get_maximal_cursor_size()
         add_gtk_version_info(capabilities, gtk, "", True)
         return capabilities
 
-    def get_ui_info(self, proto, wids):
-        info = ServerBase.get_ui_info(self, proto, wids)
+    def get_ui_info(self, *args):
+        info = ServerBase.get_ui_info(self, *args)
         info["server.root_window_size"] = self.get_root_window_size()
         return info
 
-    def do_get_info(self, proto, server_sources, window_ids):
-        info = ServerBase.do_get_info(self, proto, server_sources, window_ids)
+    def do_get_info(self, proto, *args):
+        info = ServerBase.do_get_info(self, proto, *args)
         ss = self._server_sources.get(proto)
         add_gtk_version_info(info, gtk, "server.", (ss is None) or ss.namespace)
         info["server.type"] = "Python/gtk-x11"
@@ -114,7 +112,7 @@ class GTKServerBase(ServerBase):
         pass
 
 
-    def _move_pointer(self, pos):
+    def _move_pointer(self, wid, pos):
         x, y = pos
         display = gtk.gdk.display_get_default()
         display.warp_pointer(display.get_default_screen(), x, y)
@@ -138,8 +136,31 @@ class GTKServerBase(ServerBase):
     def _process_configure_window(self, proto, packet):
         log.info("_process_configure_window(%s, %s)", proto, packet)
 
-    def _process_move_window(self, proto, packet):
-        log.info("_process_move_window(%s, %s)", proto, packet)
 
-    def _process_resize_window(self, proto, packet):
-        log.info("_process_resize_window(%s, %s)", proto, packet)
+    def send_clipboard_packet(self, *parts):
+        #overriden so we can inject the nesting check:
+        if self.clipboard_nesting_check(self._clipboard_client):
+            ServerBase.send_clipboard_packet(self, *parts)
+
+    def process_clipboard_packet(self, ss, packet):
+        #overriden so we can inject the nesting check:
+        def do_check():
+            if self.clipboard_nesting_check(ss):
+                ServerBase.process_clipboard_packet(self, ss, packet)
+        #the nesting check calls gtk, so we must call it from the main thread:
+        self.idle_add(do_check)
+
+    def clipboard_nesting_check(self, ss):
+        log("clipboard_nesting_check(%s)", ss)
+        if self._clipboard_client is None or not self._clipboard_client.clipboard_enabled:
+            return False
+        if gtk.main_level()>=10:
+            log.warn("loop nesting too deep: %s", gtk.main_level())
+            log.warn("you may have a clipboard forwarding loop, disabling the clipboard")
+            #turn off clipboard at our end:
+            self.set_clipboard_enabled_status(ss, False)
+            #if we can, tell the client to do the same:
+            if ss.clipboard_set_enabled:
+                ss.send_clipboard_enabled("probable clipboard loop detected")
+            return  False
+        return True
