@@ -1,18 +1,20 @@
 # coding=utf8
 # This file is part of Xpra.
-# Copyright (C) 2012-2013 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2012-2014 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import socket
 
 from xpra.log import Logger
-log = Logger()
+log = Logger("shadow")
 
 from xpra.net.protocol import Compressed
 from xpra.server.batch_config import DamageBatchConfig
 from xpra.util import AdHocStruct
 
+DEFAULT_DELAY = 50              #50ms refresh
+
 
 class RootWindowModel(object):
 
@@ -31,15 +33,18 @@ class RootWindowModel(object):
     def has_alpha(self):
         return False
 
+    def uses_XShm(self):
+        return False
+
+    def is_shadow(self):
+        return True
+
     def acknowledge_changes(self):
         pass
 
     def get_image(self, x, y, width, height):
         raise NotImplementedError()
 
-    def get_client_contents(self):
-        return self.window
-
     def get_property_names(self):
         return ("title", "client-machine", "window-type", "size-hints")
 
@@ -94,14 +99,18 @@ class ShadowServerBase(object):
         DamageBatchConfig.MIN_DELAY = 50            #never lower than 50ms
         DamageBatchConfig.RECALCULATE_DELAY = 0.1   #re-compute delay 10 times per second at most
 
+    def get_server_mode(self):
+        return "shadow"
+
+
     def watch_keymap_changes(self):
         pass
 
     def calculate_workarea(self):
         pass
 
-    def start_refresh(self):
-        self.timeout_add(50, self.refresh)
+    def start_refresh(self, delay=DEFAULT_DELAY):
+        self.timeout_add(delay, self.refresh)
 
     def timeout_add(self, *args):
         #usually done via gobject
@@ -115,15 +124,15 @@ class ShadowServerBase(object):
         self._damage(self.root_window_model, 0, 0, w, h)
         return True
 
-    def sanity_checks(self, proto, capabilities):
-        server_uuid = capabilities.get("server_uuid")
+    def sanity_checks(self, proto, c):
+        server_uuid = c.strget("server_uuid")
         if server_uuid:
             if server_uuid==self.uuid:
                 log.warn("Warning: shadowing your own display can be quite confusing")
-                clipboard = self._clipboard_helper and capabilities.get("clipboard", True)
+                clipboard = self._clipboard_helper and c.boolget("clipboard", True)
                 if clipboard:
                     log.warn("clipboard sharing cannot be enabled! (consider using the --no-clipboard option)")
-                    capabilities["clipboard"] = False
+                    c["clipboard"] = False
             else:
                 log.warn("This client is running within the Xpra server %s", server_uuid)
         return True
@@ -195,20 +204,6 @@ class ShadowServerBase(object):
         if len(packet)>=7:
             self._set_client_properties(proto, wid, self.root_window_model, packet[6])
 
-    def _process_move_window(self, proto, packet):
-        wid, x, y = packet[1:4]
-        self._process_window_common(wid)
-        assert self.mapped_at
-        w, h = self.mapped_at[2:4]
-        self.mapped_at = x, y, w, h
-
-    def _process_resize_window(self, proto, packet):
-        wid = packet[1]
-        self._process_window_common(wid)
-        #wid, w, h = packet[1:4]
-        #window = self._process_window_common(wid)
-        #self._cancel_damage(wid, window)
-        #self._damage(window, 0, 0, w, h)
 
     def _process_close_window(self, proto, packet):
         wid = packet[1]
