@@ -1,17 +1,17 @@
 #!/usr/bin/env python
 # This file is part of Xpra.
-# Copyright (C) 2010-2013 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2010-2014 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import sys, os, time
 
-from xpra.sound.sound_pipeline import SoundPipeline, debug
+from xpra.sound.sound_pipeline import SoundPipeline
 from xpra.sound.pulseaudio_util import has_pa
 from xpra.sound.gstreamer_util import plugin_str, get_decoder_parser, MP3, CODECS, gst
 from xpra.os_util import thread
 from xpra.log import Logger
-log = Logger()
+log = Logger("sound")
 
 
 SINKS = ["autoaudiosink"]
@@ -99,17 +99,17 @@ class SoundSink(SoundPipeline):
 
     def queue_pushing(self, *args):
         ltime = int(self.queue.get_property("current-level-time")/MS_TO_NS)
-        debug("sound sink queue pushing: level=%s", ltime)
+        log("sound sink queue pushing: level=%s", ltime)
         self.queue_state = "pushing"
 
     def queue_running(self, *args):
         ltime = int(self.queue.get_property("current-level-time")/MS_TO_NS)
-        debug("sound sink queue running: level=%s", ltime)
+        log("sound sink queue running: level=%s", ltime)
         self.queue_state = "running"
 
     def queue_underrun(self, *args):
         ltime = int(self.queue.get_property("current-level-time")/MS_TO_NS)
-        debug("sound sink queue underrun: level=%s", ltime)
+        log("sound sink queue underrun: level=%s", ltime)
         self.queue_state = "underrun"
 
     def queue_overrun(self, *args):
@@ -118,9 +118,9 @@ class SoundSink(SoundPipeline):
         #no overruns for the first 2 seconds:
         elapsed = time.time()-self.start_time
         if elapsed<2.0 or ltime<(QUEUE_TIME/MS_TO_NS/2*75/100):
-            debug("sound sink queue overrun ignored: level=%s, elapsed time=%.1f", ltime, elapsed)
+            log("sound sink queue overrun ignored: level=%s, elapsed time=%.1f", ltime, elapsed)
             return
-        debug("sound sink queue overrun: level=%s", ltime)
+        log("sound sink queue overrun: level=%s", ltime)
         self.overruns += 1
         self.emit("overrun", ltime)
 
@@ -130,7 +130,7 @@ class SoundSink(SoundPipeline):
         self.src = None
 
     def eos(self):
-        debug("eos()")
+        log("eos()")
         if self.src:
             self.src.emit('end-of-stream')
         self.cleanup()
@@ -157,7 +157,7 @@ class SoundSink(SoundPipeline):
             d = metadata.get("duration")
             if d is not None:
                 buf.duration = d
-        debug("add_data(..) queue_state=%s", self.queue_state)
+        log("add_data(..) queue_state=%s", self.queue_state)
         self.push_buffer(buf)
 
     def push_buffer(self, buf):
@@ -175,81 +175,82 @@ class SoundSink(SoundPipeline):
         self.buffer_count += 1
         self.byte_count += len(buf.data)
         ltime = int(self.queue.get_property("current-level-time")/MS_TO_NS)
-        debug("sound sink: pushed %s bytes, new buffer level: %sms", len(buf.data), ltime)
+        log("sound sink: pushed %s bytes, new buffer level: %sms", len(buf.data), ltime)
         return True
 
 
 
 def main():
-    import os.path
-    import gobject
-    if len(sys.argv) not in (2, 3):
-        print("usage: %s filename [codec]" % sys.argv[0])
-        sys.exit(1)
-        return
-    filename = sys.argv[1]
-    if not os.path.exists(filename):
-        print("file %s does not exist" % filename)
-        sys.exit(2)
-        return
-    if len(sys.argv)==3:
-        codec = sys.argv[2]
-        if codec not in CODECS:
-            print("invalid codec: %s" % codec)
-            sys.exit(2)
-            return
-    else:
-        codec = None
-        parts = filename.split(".")
-        if len(parts)>1:
-            extension = parts[-1]
-            if extension.lower() in CODECS:
-                codec = extension.lower()
-                print("guessed codec %s from file extension %s" % (codec, extension))
-        if codec is None:
-            print("assuming this is an mp3 file...")
-            codec = MP3
+    from xpra.platform import init, clean
+    init("Sound-Record")
+    try:
+        import os.path
+        import gobject
+        if len(sys.argv) not in (2, 3):
+            print("usage: %s filename [codec]" % sys.argv[0])
+            return 1
+        filename = sys.argv[1]
+        if not os.path.exists(filename):
+            print("file %s does not exist" % filename)
+            return 2
+        if len(sys.argv)==3:
+            codec = sys.argv[2]
+            if codec not in CODECS:
+                print("invalid codec: %s" % codec)
+                return 2
+        else:
+            codec = None
+            parts = filename.split(".")
+            if len(parts)>1:
+                extension = parts[-1]
+                if extension.lower() in CODECS:
+                    codec = extension.lower()
+                    print("guessed codec %s from file extension %s" % (codec, extension))
+            if codec is None:
+                print("assuming this is an mp3 file...")
+                codec = MP3
 
-    import logging
-    logging.basicConfig(format="%(asctime)s %(message)s")
-    logging.root.setLevel(logging.DEBUG)
-    f = open(filename, "rb")
-    data = f.read()
-    f.close()
-    print("loaded %s bytes from %s" % (len(data), filename))
-    #force no leak since we push all the data at once
-    global QUEUE_LEAK, GST_QUEUE_NO_LEAK, QUEUE_SILENT
-    QUEUE_LEAK = GST_QUEUE_NO_LEAK
-    QUEUE_SILENT = 1
-    ss = SoundSink(codec=codec)
-    ss.add_data(data)
-    def eos(*args):
-        print("eos")
-        gobject.idle_add(gobject_mainloop.quit)
-    ss.connect("eos", eos)
-    ss.start()
+        log.enable_debug()
+        f = open(filename, "rb")
+        data = f.read()
+        f.close()
+        print("loaded %s bytes from %s" % (len(data), filename))
+        #force no leak since we push all the data at once
+        global QUEUE_LEAK, GST_QUEUE_NO_LEAK, QUEUE_SILENT
+        QUEUE_LEAK = GST_QUEUE_NO_LEAK
+        QUEUE_SILENT = 1
+        ss = SoundSink(codec=codec)
+        ss.add_data(data)
+        def eos(*args):
+            print("eos")
+            gobject.idle_add(gobject_mainloop.quit)
+        ss.connect("eos", eos)
+        ss.start()
 
-    gobject_mainloop = gobject.MainLoop()
-    gobject.threads_init()
+        gobject_mainloop = gobject.MainLoop()
+        gobject.threads_init()
 
-    import signal
-    def deadly_signal(*args):
-        gobject.idle_add(gobject_mainloop.quit)
-    signal.signal(signal.SIGINT, deadly_signal)
-    signal.signal(signal.SIGTERM, deadly_signal)
+        import signal
+        def deadly_signal(*args):
+            gobject.idle_add(gobject_mainloop.quit)
+        signal.signal(signal.SIGINT, deadly_signal)
+        signal.signal(signal.SIGTERM, deadly_signal)
 
-    def check_for_end(*args):
-        qtime = int(ss.queue.get_property("current-level-time")/MS_TO_NS)
-        if qtime<=0:
-            log.info("underrun (end of stream)")
-            thread.start_new_thread(ss.stop, ())
-            gobject.timeout_add(500, gobject_mainloop.quit)
-            return False
-        return True
-    gobject.timeout_add(1000, check_for_end)
+        def check_for_end(*args):
+            qtime = int(ss.queue.get_property("current-level-time")/MS_TO_NS)
+            if qtime<=0:
+                log.info("underrun (end of stream)")
+                thread.start_new_thread(ss.stop, ())
+                gobject.timeout_add(500, gobject_mainloop.quit)
+                return False
+            return True
+        gobject.timeout_add(1000, check_for_end)
 
-    gobject_mainloop.run()
+        gobject_mainloop.run()
+        return 0
+    finally:
+        clean()
 
 
 if __name__ == "__main__":
-    main()
+    sys.exit(main())
