@@ -1,11 +1,12 @@
 # This file is part of Xpra.
-# Copyright (C) 2011-2013 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2011-2014 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
+import sys
 from xpra import __version__ as local_version
 from xpra.log import Logger
-log = Logger()
+log = Logger("util")
 
 def version_as_numbers(version):
     return [int(x) for x in version.split(".")]
@@ -39,16 +40,46 @@ def add_version_info(props, version_prefix=""):
     try:
         from xpra.src_info import LOCAL_MODIFICATIONS, REVISION
         from xpra.build_info import BUILD_DATE, BUILT_BY, BUILT_ON, BUILD_BIT, BUILD_CPU
-        props["build.local_modifications"] = LOCAL_MODIFICATIONS
-        props["build.date"] = BUILD_DATE
-        props["build.by"] = BUILT_BY
-        props["build.on"] = BUILT_ON
-        props["build.bit"] = BUILD_BIT
-        props["build.cpu"] = BUILD_CPU
-        props["build.revision"] = REVISION
+        props[version_prefix+"build.local_modifications"] = LOCAL_MODIFICATIONS
+        props[version_prefix+"build.date"] = BUILD_DATE
+        props[version_prefix+"build.by"] = BUILT_BY
+        props[version_prefix+"build.on"] = BUILT_ON
+        props[version_prefix+"build.bit"] = BUILD_BIT
+        props[version_prefix+"build.cpu"] = BUILD_CPU
+        props[version_prefix+"build.revision"] = REVISION
     except:
         pass
 
+def do_get_platform_info():
+    from xpra.scripts.config import python_platform
+    from xpra.os_util import platform_name
+    info = {
+            "platform"           : sys.platform,
+            "platform.name"      : platform_name(sys.platform, python_platform.release()),
+            "platform.release"   : python_platform.release(),
+            "platform.platform"  : python_platform.platform(),
+            "platform.machine"   : python_platform.machine(),
+            "platform.processor" : python_platform.processor(),
+            }
+    if sys.platform.startswith("linux") and hasattr(python_platform, "linux_distribution"):
+        info["platform.linux_distribution"] = python_platform.linux_distribution()
+    return info
+#cache the output:
+platform_info_cache = None
+def get_platform_info_cache():
+    global platform_info_cache
+    if platform_info_cache is None:
+        platform_info_cache = do_get_platform_info()
+    return platform_info_cache
+
+
+def get_platform_info(prefix=""):
+    info = {}
+    for k,v in get_platform_info_cache().items():
+        info[prefix+k] = v
+    return info
+
+
 def main():
     d = {}
     add_version_info(d)
