@@ -1,6 +1,6 @@
 # This file is part of Xpra.
 # Copyright (C) 2008, 2009 Nathaniel Smith <njs@pobox.com>
-# Copyright (C) 2012, 2013 Antoine Martin <antoine@devloop.org.uk>
+# Copyright (C) 2012-2014 Antoine Martin <antoine@devloop.org.uk>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
@@ -19,18 +19,22 @@ from xpra.x11.gtk_x11.gdk_bindings import (
                 get_xvisual,                #@UnresolvedImport
                )
 from xpra.x11.bindings.window_bindings import (
-                const,                      #@UnresolvedImport
+                constants,                      #@UnresolvedImport
                 X11WindowBindings,          #@UnresolvedImport
-                NoSuchProperty,             #@UnresolvedImport
                 PropertyError)              #@UnresolvedImport
-from xpra.codecs.argb.argb import premultiply_argb_in_place #@UnresolvedImport
 X11Window = X11WindowBindings()
 
 from xpra.os_util import StringIOClass
 from xpra.x11.xsettings_prop import set_settings, get_settings
 from xpra.x11.gtk_x11.error import trap, XError
 from xpra.log import Logger
-log = Logger()
+log = Logger("x11", "window")
+
+try:
+    from xpra.codecs.argb.argb import premultiply_argb_in_place #@UnresolvedImport
+except:
+    log.warn("argb module is missing, icon transparency will be incorrect")
+    premultiply_argb_in_place = None
 
 import sys
 if sys.version > '3':
@@ -38,6 +42,19 @@ if sys.version > '3':
     unicode = str           #@ReservedAssignment
 
 
+PMaxSize        = constants["PMaxSize"]
+PMinSize        = constants["PMinSize"]
+PBaseSize       = constants["PBaseSize"]
+PResizeInc      = constants["PResizeInc"]
+PAspect         = constants["PAspect"]
+PWinGravity     = constants["PWinGravity"]
+XUrgencyHint    = constants["XUrgencyHint"]
+WindowGroupHint = constants["WindowGroupHint"]
+StateHint       = constants["StateHint"]
+IconicState     = constants["IconicState"]
+InputHint       = constants["InputHint"]
+
+
 def unsupported(*args):
     raise Exception("unsupported")
 
@@ -55,6 +72,10 @@ class WMSizeHints(object):
     def __init__(self, disp, data):
         # pre-ICCCM size is 15
         data = _force_length("WM_SIZE_HINTS", data, 18*4, noerror_length=15*4)
+        def normint(v):
+            if v==(2**32-1):
+                return -1
+            return v
         (flags,
          pad1, pad2, pad3, pad4,            #@UnusedVariable
          min_width, min_height,
@@ -63,25 +84,25 @@ class WMSizeHints(object):
          min_aspect_num, min_aspect_denom,
          max_aspect_num, max_aspect_denom,
          base_width, base_height,
-         win_gravity) = struct.unpack("=" + "I" * 18, data) #@UnusedVariable
+         win_gravity) = [normint(x) for x in struct.unpack("=" + "I" * 18, data)] #@UnusedVariable
         # We only extract the pieces we care about:
-        if flags & const["PMaxSize"]:
+        if flags & PMaxSize:
             self.max_size = (max_width, max_height)
         else:
             self.max_size = None
-        if flags & const["PMinSize"]:
+        if flags & PMinSize:
             self.min_size = (min_width, min_height)
         else:
             self.min_size = None
-        if flags & const["PBaseSize"]:
+        if flags & PBaseSize:
             self.base_size = (base_width, base_height)
         else:
             self.base_size = None
-        if flags & const["PResizeInc"]:
+        if flags & PResizeInc:
             self.resize_inc = (width_inc, height_inc)
         else:
             self.resize_inc = None
-        if flags & const["PAspect"]:
+        if flags & PAspect:
             self.min_aspect = min_aspect_num * 1.0 / min_aspect_denom
             self.min_aspect_ratio = (min_aspect_num, min_aspect_denom)
             self.max_aspect = max_aspect_num * 1.0 / max_aspect_denom
@@ -89,7 +110,7 @@ class WMSizeHints(object):
         else:
             self.min_aspect, self.max_aspect = (None, None)
             self.min_aspect_ratio, self.max_aspect_ratio = (None, None)
-        if flags & const["PWinGravity"]:
+        if flags & PWinGravity:
             self.win_gravity = win_gravity
         else:
             self.win_gravity = -1
@@ -129,8 +150,8 @@ class WMHints(object):
          window_group) = struct.unpack("=" + "i" * 9, data)
         # NB the last field is missing from at least some ICCCM 2.0's (typo).
         # FIXME: extract icon stuff too
-        self.urgency = bool(flags & const["XUrgencyHint"])
-        if flags & const["WindowGroupHint"]:
+        self.urgency = bool(flags & XUrgencyHint)
+        if flags & WindowGroupHint:
             try:
                 pywin = get_pywindow(disp, window_group)
             except:
@@ -138,11 +159,11 @@ class WMHints(object):
             self.group_leader = (window_group, pywin)
         else:
             self.group_leader = None
-        if flags & const["StateHint"]:
-            self.start_iconic = (initial_state == const["IconicState"])
+        if flags & StateHint:
+            self.start_iconic = (initial_state == IconicState)
         else:
             self.start_iconic = None
-        if flags & const["InputHint"]:
+        if flags & InputHint:
             self.input = _input
         else:
             self.input = None
@@ -204,7 +225,8 @@ def _read_image(disp, stream):
     # Cairo uses premultiplied alpha. EWMH actually doesn't specify what it
     # uses, but apparently the de-facto standard is non-premultiplied. (At
     # least that's what Compiz's sources say.)
-    premultiply_argb_in_place(surf.get_data())
+    if premultiply_argb_in_place:
+        premultiply_argb_in_place(surf.get_data())
     return (width * height, surf)
 
 # This returns a cairo ImageSurface which contains the largest icon defined in
@@ -360,10 +382,11 @@ def prop_get(target, key, etype, ignore_errors=False, raise_xerrors=False):
         scalar_type = etype
     (_, atom, _, _, _, _) = _prop_types[scalar_type]
     try:
-        data = trap.call_synced(X11Window.XGetWindowProperty, get_xwindow(target), key, atom)
-    except NoSuchProperty:
-        log.debug("Missing property %s (%s)", key, etype)
-        return None
+        data = trap.call_synced(X11Window.XGetWindowProperty, get_xwindow(target), key, atom, etype)
+        if data is None:
+            if not ignore_errors:
+                log("Missing property %s (%s)", key, etype)
+            return None
     except XError:
         if raise_xerrors:
             raise
