@@ -8,7 +8,7 @@ import gtk.gdk
 
 from xpra.x11.x11_server_base import X11ServerBase
 from xpra.server.shadow_server_base import ShadowServerBase
-from xpra.server.gtk_shadow_server_base import GTKRootWindowModel
+from xpra.server.gtk_root_window_model import GTKRootWindowModel
 
 
 class ShadowX11Server(ShadowServerBase, X11ServerBase):
@@ -17,8 +17,8 @@ class ShadowX11Server(ShadowServerBase, X11ServerBase):
         ShadowServerBase.__init__(self, gtk.gdk.get_default_root_window())
         X11ServerBase.__init__(self)
 
-    def init(self, sockets, opts):
-        X11ServerBase.init(self, False, sockets, opts)
+    def init(self, opts):
+        X11ServerBase.init(self, False, opts)
 
     def makeRootWindowModel(self):
         return GTKRootWindowModel(self.root)
